# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////
from typing import List, Dict, Any, Optional

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import (
    Qt,
    QSize,
    Signal,
)

from PySide6.QtWidgets import (
    QWidget,
    QFrame,
    QVBoxLayout,
    QHBoxLayout,
    QLabel,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# /////////////////////////////////////////////////////////////////////////////////////////////
from ...kernel.app_components import *
from ...kernel.app_resources import *
from ...kernel.app_settings import Settings

# Import lazy pour éviter l'import circulaire
# from ...kernel import Kernel

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class SettingsPanel(QFrame):
    """
    This class is used to create a settings panel.
    It contains a top border, a content settings frame and a theme settings container.
    The settings panel is used to display the settings.
    """

    _widgets: List = []  # Type hint removed to avoid circular import
    _settings: Dict[str, Any] = {}  # Stockage des paramètres

    # Signal émis quand un paramètre change
    settingChanged = Signal(str, object)  # key, value
    # Signal émis quand la langue change
    languageChanged = Signal()

    # ///////////////////////////////////////////////////////////////

    def __init__(
        self, parent: QWidget = None, width: int = 240, load_from_yaml: bool = True
    ) -> None:
        super(SettingsPanel, self).__init__(parent)

        # ///////////////////////////////////////////////////////////////
        # Store configuration
        self._width = width

        self.setObjectName("settingsPanel")
        self.setMinimumSize(QSize(0, 0))
        self.setMaximumSize(QSize(0, 16777215))
        self.setFrameShape(QFrame.NoFrame)
        self.setFrameShadow(QFrame.Raised)
        # //////
        self.VL_settingsPanel = QVBoxLayout(self)
        self.VL_settingsPanel.setSpacing(0)
        self.VL_settingsPanel.setObjectName("VL_settingsPanel")
        self.VL_settingsPanel.setContentsMargins(0, 0, 0, 0)

        # ///////////////////////////////////////////////////////////////

        self.settingsTopBorder = QFrame(self)
        self.settingsTopBorder.setObjectName("settingsTopBorder")
        self.settingsTopBorder.setMaximumSize(QSize(16777215, 3))
        self.settingsTopBorder.setFrameShape(QFrame.NoFrame)
        self.settingsTopBorder.setFrameShadow(QFrame.Raised)
        #
        self.VL_settingsPanel.addWidget(self.settingsTopBorder)

        # ///////////////////////////////////////////////////////////////

        self.contentSettings = QFrame(self)
        self.contentSettings.setObjectName("contentSettings")
        self.contentSettings.setFrameShape(QFrame.NoFrame)
        self.contentSettings.setFrameShadow(QFrame.Raised)
        #
        self.VL_settingsPanel.addWidget(self.contentSettings)
        # //////
        self.VL_contentSettings = QVBoxLayout(self.contentSettings)
        self.VL_contentSettings.setObjectName("VL_contentSettings")
        self.VL_contentSettings.setSpacing(0)
        self.VL_contentSettings.setContentsMargins(0, 0, 0, 0)
        self.VL_contentSettings.setAlignment(Qt.AlignTop)

        # ///////////////////////////////////////////////////////////////

        self.themeSettingsContainer = QFrame(self.contentSettings)
        self.themeSettingsContainer.setObjectName("themeSettingsContainer")
        self.themeSettingsContainer.setFrameShape(QFrame.NoFrame)
        self.themeSettingsContainer.setFrameShadow(QFrame.Raised)
        #
        self.VL_contentSettings.addWidget(self.themeSettingsContainer, 0, Qt.AlignTop)
        # //////
        self.VL_themeSettingsContainer = QVBoxLayout(self.themeSettingsContainer)
        self.VL_themeSettingsContainer.setSpacing(8)
        self.VL_themeSettingsContainer.setObjectName("VL_themeSettingsContainer")
        self.VL_themeSettingsContainer.setContentsMargins(10, 10, 10, 10)

        # ///////////////////////////////////////////////////////////////

        self.themeLabel = QLabel("Theme actif", self.themeSettingsContainer)
        self.themeLabel.setObjectName("themeLabel")
        self.themeLabel.setFont(Fonts.SEGOE_UI_10_SB)
        self.themeLabel.setAlignment(Qt.AlignLeading | Qt.AlignLeft | Qt.AlignVCenter)
        #
        self.VL_themeSettingsContainer.addWidget(self.themeLabel)

        # ///////////////////////////////////////////////////////////////

        # Lazy import to avoid circular imports
        try:
            from ezqt_widgets import OptionSelector
            from ...kernel.translation_helpers import tr

            # Créer les items traduits dès le début
            try:
                translated_items = [tr("Light"), tr("Dark")]
            except:
                # Fallback si la traduction n'est pas disponible
                translated_items = ["Light", "Dark"]

            # S'assurer que les items sont cohérents avec le mapping
            self._init_theme_mapping()

            self.themeToggleButton = OptionSelector(
                items=translated_items,
                default_id=0,  # Utiliser default_id au lieu de default
                parent=self.themeSettingsContainer,
                animation_duration=Settings.Gui.TIME_ANIMATION,
            )
            self.themeToggleButton.setObjectName("themeToggleButton")
            self.themeToggleButton.setSizePolicy(SizePolicy.H_EXPANDING_V_FIXED)
            self.themeToggleButton.setFixedHeight(40)
            self._widgets.append(self.themeToggleButton)

            # Initialiser le mapping de thème AVANT de connecter les signaux
            self._init_theme_mapping()

            # Connecter le signal de changement du sélecteur de thème
            self._connect_theme_selector_signals()

            # Initialiser le sélecteur avec la valeur par défaut depuis le YAML
            self._initialize_theme_selector_default()

            #
            self.VL_themeSettingsContainer.addWidget(self.themeToggleButton)
        except ImportError:
            print("Warning: OptionSelector not available, theme toggle not created")

        # ///////////////////////////////////////////////////////////////
        # Chargement automatique depuis YAML si demandé
        if load_from_yaml:
            self.load_settings_from_yaml()

        # Connecter les changements de paramètres
        self.settingChanged.connect(self._on_setting_changed)

    # ///////////////////////////////////////////////////////////////

    def load_settings_from_yaml(self) -> None:
        """Charge les paramètres depuis le fichier YAML."""
        try:
            # Import direct pour éviter l'import circulaire
            from ...kernel.app_functions import Kernel

            # Charger la configuration settings_panel depuis le YAML
            settings_config = Kernel.loadKernelConfig("settings_panel")

            # Créer les widgets pour chaque paramètre
            for key, config in settings_config.items():
                # Exclure le thème car il est déjà géré manuellement par OptionSelector
                if key == "theme":
                    continue

                if config.get("enabled", True):  # Vérifier si le paramètre est activé
                    widget = self.add_setting_from_config(key, config)

                    # Utiliser la valeur default du config (qui peut avoir été mise à jour)
                    default_value = config.get("default")
                    if default_value is not None:
                        widget.set_value(default_value)

        except KeyError:
            print("Warning: Section 'settings_panel' not found in YAML configuration")
        except Exception as e:
            print(f"Warning: Error loading settings from YAML: {e}")

    def add_setting_from_config(self, key: str, config: dict) -> QWidget:
        """Ajoute un paramètre basé sur sa configuration YAML."""
        setting_type = config.get("type", "text")
        label = config.get("label", key)
        description = config.get("description", "")
        default_value = config.get("default", None)

        # Créer un container pour ce paramètre (comme themeSettingsContainer)
        setting_container = QFrame(self.contentSettings)
        setting_container.setObjectName(f"settingContainer_{key}")
        setting_container.setFrameShape(QFrame.NoFrame)
        setting_container.setFrameShadow(QFrame.Raised)

        # Layout du container avec marges
        container_layout = QVBoxLayout(setting_container)
        container_layout.setSpacing(8)
        container_layout.setObjectName(f"VL_settingContainer_{key}")
        container_layout.setContentsMargins(10, 10, 10, 10)

        # Créer le widget selon le type
        if setting_type == "toggle":
            widget = self._create_toggle_widget(label, description, default_value, key)
        elif setting_type == "select":
            options = config.get("options", [])
            widget = self._create_select_widget(
                label, description, options, default_value, key
            )
        elif setting_type == "slider":
            min_val = config.get("min", 0)
            max_val = config.get("max", 100)
            unit = config.get("unit", "")
            widget = self._create_slider_widget(
                label, description, min_val, max_val, default_value, unit, key
            )
        elif setting_type == "checkbox":
            widget = self._create_checkbox_widget(
                label, description, default_value, key
            )
        else:  # text par défaut
            widget = self._create_text_widget(label, description, default_value, key)

        # Ajouter le widget au container
        container_layout.addWidget(widget)

        # Ajouter le container au layout principal
        self.VL_contentSettings.addWidget(setting_container)

        # Stocker la référence
        self._settings[key] = widget

        return widget

    def _create_toggle_widget(
        self, label: str, description: str, default: bool, key: str = None
    ) -> QWidget:
        """Crée un widget toggle avec label et description."""
        from ...widgets.extended.setting_widgets import SettingToggle

        widget = SettingToggle(label, description, default)
        if key:
            widget._key = key
        widget.valueChanged.connect(self._on_setting_changed)
        return widget

    def _create_select_widget(
        self, label: str, description: str, options: list, default: str, key: str = None
    ) -> QWidget:
        """Crée un widget select avec label et description."""
        from ...widgets.extended.setting_widgets import SettingSelect

        widget = SettingSelect(label, description, options, default)
        if key:
            widget._key = key
        widget.valueChanged.connect(self._on_setting_changed)
        return widget

    def _create_slider_widget(
        self,
        label: str,
        description: str,
        min_val: int,
        max_val: int,
        default: int,
        unit: str,
        key: str = None,
    ) -> QWidget:
        """Crée un widget slider avec label et description."""
        from ...widgets.extended.setting_widgets import SettingSlider

        widget = SettingSlider(label, description, min_val, max_val, default, unit)
        if key:
            widget._key = key
        widget.valueChanged.connect(self._on_setting_changed)
        return widget

    def _create_checkbox_widget(
        self, label: str, description: str, default: bool, key: str = None
    ) -> QWidget:
        """Crée un widget checkbox avec label et description."""
        from ...widgets.extended.setting_widgets import SettingCheckbox

        widget = SettingCheckbox(label, description, default)
        if key:
            widget._key = key
        widget.valueChanged.connect(self._on_setting_changed)
        return widget

    def _create_text_widget(
        self, label: str, description: str, default: str, key: str = None
    ) -> QWidget:
        """Crée un widget text avec label et description."""
        from ...widgets.extended.setting_widgets import SettingText

        widget = SettingText(label, description, default)
        if key:
            widget._key = key
        widget.valueChanged.connect(self._on_setting_changed)
        return widget

    # ///////////////////////////////////////////////////////////////
    # Méthodes simplifiées pour ajout manuel de paramètres

    def add_toggle_setting(
        self,
        key: str,
        label: str,
        default: bool = False,
        description: str = "",
        enabled: bool = True,
    ):
        """Ajoute un paramètre toggle."""
        from ...widgets.extended.setting_widgets import SettingToggle

        widget = SettingToggle(label, description, default)
        widget._key = key  # Définir la clé
        widget.valueChanged.connect(self._on_setting_changed)

        self._settings[key] = widget
        self.add_setting_widget(widget)
        return widget

    def add_select_setting(
        self,
        key: str,
        label: str,
        options: List[str],
        default: str = None,
        description: str = "",
        enabled: bool = True,
    ):
        """Ajoute un paramètre de sélection."""
        from ...widgets.extended.setting_widgets import SettingSelect

        widget = SettingSelect(label, description, options, default)
        widget._key = key  # Définir la clé
        widget.valueChanged.connect(self._on_setting_changed)

        self._settings[key] = widget
        self.add_setting_widget(widget)
        return widget

    def add_slider_setting(
        self,
        key: str,
        label: str,
        min_val: int,
        max_val: int,
        default: int,
        unit: str = "",
        description: str = "",
        enabled: bool = True,
    ):
        """Ajoute un paramètre slider."""
        from ...widgets.extended.setting_widgets import SettingSlider

        widget = SettingSlider(label, description, min_val, max_val, default, unit)
        widget._key = key  # Définir la clé
        widget.valueChanged.connect(self._on_setting_changed)

        self._settings[key] = widget
        self.add_setting_widget(widget)
        return widget

    def add_text_setting(
        self,
        key: str,
        label: str,
        default: str = "",
        description: str = "",
        enabled: bool = True,
    ):
        """Ajoute un paramètre texte."""
        from ...widgets.extended.setting_widgets import SettingText

        widget = SettingText(label, description, default)
        widget._key = key  # Définir la clé
        widget.valueChanged.connect(self._on_setting_changed)

        self._settings[key] = widget
        self.add_setting_widget(widget)
        return widget

    def add_checkbox_setting(
        self,
        key: str,
        label: str,
        default: bool = False,
        description: str = "",
        enabled: bool = True,
    ):
        """Ajoute un paramètre checkbox."""
        from ...widgets.extended.setting_widgets import SettingCheckbox

        widget = SettingCheckbox(label, description, default)
        widget._key = key  # Définir la clé
        widget.valueChanged.connect(self._on_setting_changed)

        self._settings[key] = widget
        self.add_setting_widget(widget)
        return widget

    def _on_setting_changed(self, key: str, value):
        """Appelé quand un paramètre change."""
        # Protection contre la récursion
        if not hasattr(self, "_processing_setting_change"):
            self._processing_setting_change = False

        if self._processing_setting_change:
            return  # Éviter la récursion

        self._processing_setting_change = True

        try:
            # Sauvegarder dans YAML
            try:
                # Import direct pour éviter l'import circulaire
                from ...kernel.app_functions import Kernel

                # Sauvegarder directement dans settings_panel[key].default
                Kernel.writeYamlConfig(["settings_panel", key, "default"], value)
            except Exception as e:
                print(f"Warning: Could not save setting '{key}' to YAML: {e}")

            # Gestion spéciale pour les changements de langue
            if key == "language":
                try:
                    from ...kernel.translation_manager import get_translation_manager

                    translation_manager = get_translation_manager()
                    # Vérifier si la langue change vraiment
                    current_lang = translation_manager.get_current_language_name()
                    if current_lang != str(value):
                        translation_manager.load_language(str(value))
                        # Émettre le signal de changement de langue
                        self.languageChanged.emit()
                except Exception as e:
                    print(f"Warning: Could not change language: {e}")

            # Émettre un signal pour l'application
            self.settingChanged.emit(key, value)
        finally:
            self._processing_setting_change = False

    # ///////////////////////////////////////////////////////////////
    # Méthodes utilitaires

    def get_setting_value(self, key: str) -> Any:
        """Récupère la valeur d'un paramètre."""
        if key in self._settings:
            return self._settings[key].get_value()
        return None

    def set_setting_value(self, key: str, value: Any) -> None:
        """Définit la valeur d'un paramètre."""
        if key in self._settings:
            self._settings[key].set_value(value)

    def get_all_settings(self) -> Dict[str, Any]:
        """Récupère tous les paramètres et leurs valeurs."""
        return {key: widget.get_value() for key, widget in self._settings.items()}

    def save_all_settings_to_yaml(self) -> None:
        """Sauvegarde tous les paramètres dans le YAML."""
        # Import direct pour éviter l'import circulaire
        from ...kernel.app_functions import Kernel

        for key, widget in self._settings.items():
            try:
                Kernel.writeYamlConfig(
                    ["settings_panel", key, "default"], widget.get_value()
                )
            except Exception as e:
                print(f"Warning: Could not save setting '{key}' to YAML: {e}")

    # ///////////////////////////////////////////////////////////////
    # Méthodes existantes (conservées pour compatibilité)

    def get_width(self) -> int:
        """Get the configured width."""
        return self._width

    def set_width(self, width: int) -> None:
        """Set the configured width."""
        self._width = width

    def get_theme_toggle_button(self):
        """Get the theme toggle button if available."""
        if hasattr(self, "themeToggleButton"):
            return self.themeToggleButton
        return None

    def update_all_theme_icons(self) -> None:
        """Update theme icons for all widgets that support it."""
        for widget in self._widgets:
            if hasattr(widget, "update_theme_icon"):
                widget.update_theme_icon()

    def _init_theme_mapping(self) -> None:
        """Initialise le mapping entre les IDs et les valeurs anglaises."""
        # Mapping simple : 0 = Light, 1 = Dark
        self._theme_mapping = {0: "Light", 1: "Dark"}

        # Mapping inverse pour la compatibilité
        self._theme_mapping_reverse = {"Light": 0, "Dark": 1, "light": 0, "dark": 1}

    def _connect_theme_selector_signals(self) -> None:
        """Connecte les signaux du sélecteur de thème."""
        try:
            if hasattr(self, "themeToggleButton"):
                # Connecter le signal valueChanged du OptionSelector
                theme_button = self.themeToggleButton

                if hasattr(theme_button, "valueChanged"):
                    theme_button.valueChanged.connect(self._on_theme_selector_changed)
                elif hasattr(theme_button, "clicked"):
                    theme_button.clicked.connect(self._on_theme_selector_clicked)
        except Exception as e:
            pass

    def _on_theme_selector_changed(self, value):
        """Appelé quand le sélecteur de thème change."""
        try:
            # La nouvelle version d'OptionSelector émet directement la valeur textuelle
            # Convertir la valeur en ID puis en valeur anglaise
            theme_id = self._get_theme_id_from_value(value)
            english_value = self._theme_mapping.get(theme_id, "light")

            # Sauvegarder la valeur anglaise dans le YAML
            from ...kernel.app_functions import Kernel

            Kernel.writeYamlConfig(
                ["settings_panel", "theme", "default"], english_value.lower()
            )

            # Émettre le signal avec la valeur anglaise
            self.settingChanged.emit("theme", english_value.lower())

        except Exception as e:
            print(f"Warning: Could not handle theme selector change: {e}")

    def _on_theme_selector_clicked(self):
        """Appelé quand le sélecteur de thème est cliqué."""
        try:
            if hasattr(self, "themeToggleButton"):
                # Récupérer la valeur actuelle (ID)
                current_id = self.themeToggleButton.value_id
                english_value = self._theme_mapping.get(current_id, "light")

                # Sauvegarder la valeur anglaise dans le YAML
                from ...kernel.app_functions import Kernel

                Kernel.writeYamlConfig(
                    ["settings_panel", "theme", "default"], english_value.lower()
                )

                # Émettre le signal avec la valeur anglaise
                self.settingChanged.emit("theme", english_value.lower())

        except Exception as e:
            print(f"Warning: Could not handle theme selector click: {e}")

    def _get_theme_id_from_value(self, value: str) -> int:
        """Convertit une valeur textuelle en ID de thème."""
        # S'assurer que le mapping existe
        if (
            not hasattr(self, "_theme_mapping_reverse")
            or self._theme_mapping_reverse is None
        ):
            self._init_theme_mapping()

        # Chercher dans le mapping inverse
        return self._theme_mapping_reverse.get(value, 0)  # 0 = Light par défaut

    def _initialize_theme_selector_default(self) -> None:
        """Initialise le sélecteur de thème avec la valeur par défaut depuis le YAML."""
        try:
            if hasattr(self, "themeToggleButton"):
                # Charger la valeur par défaut depuis le YAML
                from ...kernel.app_functions import Kernel

                settings_panel = Kernel.loadKernelConfig("settings_panel")
                default_theme = settings_panel.get("theme", {}).get("default", "dark")

                # Convertir en ID
                theme_id = 0 if default_theme.lower() == "light" else 1

                # Initialiser le sélecteur
                if hasattr(self.themeToggleButton, "initialize_selector"):
                    self.themeToggleButton.initialize_selector(theme_id)
                elif hasattr(self.themeToggleButton, "value_id"):
                    self.themeToggleButton.value_id = theme_id

        except Exception as e:
            # Ignorer les erreurs
            pass

    def update_theme_selector_items(self) -> None:
        """Met à jour les items du sélecteur de thème avec les traductions."""
        try:
            if hasattr(self, "themeToggleButton"):
                from ...kernel.translation_helpers import tr

                # Mettre à jour le mapping avec les nouvelles traductions
                self._init_theme_mapping()

                # Traduire les items pour l'affichage
                translated_items = [tr("Light"), tr("Dark")]

                # Sauvegarder la valeur actuelle (ID)
                theme_button = self.themeToggleButton
                current_id = (
                    theme_button.value_id if hasattr(theme_button, "value_id") else 0
                )

                # La nouvelle version d'OptionSelector gère automatiquement les traductions
                # Il suffit de mettre à jour les items et de restaurer l'ID
                if hasattr(theme_button, "items"):
                    theme_button.items = translated_items
                    # Restaurer l'ID actuel
                    theme_button.value_id = current_id

        except Exception as e:
            # Ignorer les erreurs
            pass

    def add_setting_widget(self, widget: QWidget) -> None:
        """Add a new setting widget to the settings panel."""
        # Créer un container pour le paramètre (comme themeSettingsContainer)
        setting_container = QFrame(self.contentSettings)
        setting_container.setObjectName(f"settingContainer_{widget.objectName()}")
        setting_container.setFrameShape(QFrame.NoFrame)
        setting_container.setFrameShadow(QFrame.Raised)

        # Layout du container avec marges (comme VL_themeSettingsContainer)
        container_layout = QVBoxLayout(setting_container)
        container_layout.setSpacing(8)
        container_layout.setContentsMargins(10, 10, 10, 10)

        # Ajouter le widget au container
        container_layout.addWidget(widget)

        # Ajouter le container au layout principal
        self.VL_contentSettings.addWidget(setting_container)
        self._widgets.append(widget)

    def add_setting_section(self, title: str = "") -> QFrame:
        """Add a new settings section with optional title."""
        section = QFrame(self.contentSettings)
        section.setObjectName(f"settingsSection_{title.replace(' ', '_')}")
        section.setFrameShape(QFrame.NoFrame)
        section.setFrameShadow(QFrame.Raised)

        section_layout = QVBoxLayout(section)
        section_layout.setSpacing(8)
        section_layout.setContentsMargins(10, 10, 10, 10)

        if title:
            title_label = QLabel(title, section)
            title_label.setFont(Fonts.SEGOE_UI_10_REG)
            title_label.setAlignment(Qt.AlignLeading | Qt.AlignLeft | Qt.AlignVCenter)
            section_layout.addWidget(title_label)

        self.VL_contentSettings.addWidget(section)
        return section



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# WIDGETS CORE
from .widgets.core import *

# WIDGETS EXTENDED
from .widgets.extended import *

# WIDGETS CUSTOM GRIPS
from .widgets.custom_grips import *



# -*- coding: utf-8 -*-
"""
Gestionnaire de traduction pour EzQt_App
"""

from PySide6.QtCore import QTranslator, QCoreApplication, QLocale, Signal, QObject
from pathlib import Path
import os
import sys
from colorama import Fore, Style

# Import APP_PATH depuis app_functions
from .app_functions import APP_PATH


class TranslationManager(QObject):
    """Gestionnaire de traduction pour EzQt_App"""

    languageChanged = Signal(str)

    def __init__(self):
        super().__init__()
        self.translator = QTranslator()
        self.current_language = "en"

        # Système de retraduction automatique
        self._translatable_widgets = []  # Liste des widgets à retraduire
        self._translatable_texts = {}  # {widget: original_text}

        # Traductions .ts chargées directement
        self._ts_translations = {}

        # Déterminer le chemin des traductions
        if hasattr(sys, "_MEIPASS"):
            # Mode frozen (exécutable)
            self.translations_dir = (
                Path(sys._MEIPASS) / "ezqt_app" / "resources" / "translations"
            )
        else:
            # Mode développement - chercher dans l'ordre de priorité
            # Utiliser la même logique que makeRequiredFiles
            project_path = Path.cwd()  # Même logique que Helper.Maker(Path.cwd())
            possible_paths = [
                project_path
                / "bin"
                / "translations",  # Projet utilisateur (priorité 1)
                APP_PATH / "bin" / "translations",  # APP_PATH (priorité 2)
                Path(__file__).parent.parent
                / "resources"
                / "translations",  # Développement local (priorité 3)
            ]

            # Essayer de récupérer depuis le package installé si pas trouvé
            try:
                package_translations = self._get_package_translations_dir()
                if package_translations.exists():
                    possible_paths.append(
                        package_translations
                    )  # Package installé (priorité 4)
            except Exception as e:
                # print(f"⚠️ Impossible de récupérer les traductions du package: {e}")
                pass

            # Chercher le premier dossier qui existe
            self.translations_dir = None
            for path in possible_paths:
                if path.exists():
                    self.translations_dir = path
                    break

            # Si aucun dossier trouvé, créer le dossier du projet utilisateur
            if self.translations_dir is None:
                self.translations_dir = project_path / "bin" / "translations"
                self.translations_dir.mkdir(parents=True, exist_ok=True)

        # Mapping des noms de langue vers les codes
        self.language_mapping = {
            "English": "en",
            "Français": "fr",
            "Español": "es",
            "Deutsch": "de",
        }

        # Mapping inverse pour l'affichage
        self.language_names = {
            "en": "English",
            "fr": "Français",
            "es": "Español",
            "de": "Deutsch",
        }

        print(
            Fore.LIGHTBLUE_EX
            + f"+ [AppKernel] | Translation system initialized."
            + Style.RESET_ALL
        )

    def _get_package_translations_dir(self) -> Path:
        """Récupère le dossier des traductions depuis le package installé"""
        try:
            import pkg_resources

            # Utiliser la même méthode que Kernel.getPackageResource
            package_translations = Path(
                pkg_resources.resource_filename("ezqt_app", "resources/translations")
            )
            return package_translations
        except Exception as e:
            return Path(__file__).parent.parent / "resources" / "translations"

    def _load_ts_file(self, ts_file_path: Path) -> bool:
        """Charge un fichier .ts directement en parsant le XML"""
        try:
            import xml.etree.ElementTree as ET

            # Parser le fichier XML
            tree = ET.parse(ts_file_path)
            root = tree.getroot()

            # Extraire les traductions
            translations = {}
            for message in root.findall(".//message"):
                source = message.find("source")
                translation = message.find("translation")

                if source is not None and translation is not None:
                    source_text = source.text.strip() if source.text else ""
                    translation_text = (
                        translation.text.strip() if translation.text else ""
                    )

                    if source_text and translation_text:
                        translations[source_text] = translation_text

            # Stocker les traductions dans le gestionnaire
            self._ts_translations = translations
            print(
                Fore.LIGHTBLUE_EX
                + f"+ [AppKernel] | Translations loaded from {ts_file_path.name} : {len(translations)} entries."
                + Style.RESET_ALL
            )
            return True

        except Exception as e:
            return False

    def load_language(self, language_name: str) -> bool:
        """Charge une langue par son nom"""
        language_code = self.language_mapping.get(language_name, "en")
        return self.load_language_by_code(language_code)

    def load_language_by_code(self, language_code: str) -> bool:
        """Charge une langue par son code"""
        # Vérifier si la langue est déjà chargée
        if self.current_language == language_code:
            return True

        # Retirer l'ancien traducteur
        QCoreApplication.removeTranslator(self.translator)

        # Créer un nouveau traducteur
        self.translator = QTranslator()

        # Essayer plusieurs formats de fichiers (priorité aux .ts)
        possible_files = [
            self.translations_dir / f"ezqt_app_{language_code}.ts",  # Priorité 1: .ts
            self.translations_dir
            / f"{language_code}.ts",  # Priorité 2: .ts sans préfixe
            self.translations_dir
            / f"ezqt_app_{language_code}.qm",  # Priorité 3: .qm (si .ts pas trouvé)
            self.translations_dir
            / f"{language_code}.qm",  # Priorité 4: .qm sans préfixe
        ]

        translation_file = None
        for file_path in possible_files:
            if file_path.exists():
                translation_file = file_path
                break

        if translation_file:
            # Pour les fichiers .ts, on peut les parser directement
            if translation_file.suffix == ".ts":
                success = self._load_ts_file(translation_file)
                if success:
                    self.current_language = language_code
                    print(
                        Fore.LIGHTBLUE_EX
                        + f"+ [AppKernel] | Language loaded : {self.language_names.get(language_code, language_code)}."
                        + Style.RESET_ALL
                    )

                    # Retraduire tous les widgets enregistrés
                    self._retranslate_all_widgets()

                    # Émettre le signal de changement de langue
                    self.languageChanged.emit(language_code)
                    return True
                return False

            if self.translator.load(str(translation_file)):
                QCoreApplication.installTranslator(self.translator)
                self.current_language = language_code
                print(
                    Fore.LIGHTBLUE_EX
                    + f"+ [AppKernel] | Language loaded : {self.language_names.get(language_code, language_code)}."
                    + Style.RESET_ALL
                )

                # Retraduire tous les widgets enregistrés
                self._retranslate_all_widgets()

                # Émettre le signal de changement de langue
                self.languageChanged.emit(language_code)
                return True
            else:
                pass
        else:
            pass

        return False

    def get_available_languages(self) -> list:
        """Retourne la liste des langues disponibles"""
        languages = []

        # Vérifier que le dossier existe
        if not self.translations_dir or not self.translations_dir.exists():
            # Retourner au moins l'anglais par défaut
            return ["English"]

        # Chercher les fichiers .ts et .qm (priorité aux .ts)
        try:
            # Fichiers .ts (priorité 1)
            for ts_file in self.translations_dir.glob("ezqt_app_*.ts"):
                code = ts_file.stem.split("_")[-1]
                if code in self.language_names:
                    languages.append(self.language_names[code])

            # Fichiers .qm (si pas déjà trouvé en .ts)
            for qm_file in self.translations_dir.glob("ezqt_app_*.qm"):
                code = qm_file.stem.split("_")[-1]
                if (
                    code in self.language_names
                    and self.language_names[code] not in languages
                ):
                    languages.append(self.language_names[code])
        except Exception as e:
            pass

        # Ajouter l'anglais par défaut s'il n'est pas déjà présent
        if "English" not in languages:
            languages.append("English")

        return languages

    def get_current_language_name(self) -> str:
        """Retourne le nom de la langue actuelle"""
        return self.language_names.get(self.current_language, "English")

    def get_current_language_code(self) -> str:
        """Retourne le code de la langue actuelle"""
        return self.current_language

    def translate(self, text: str) -> str:
        """Traduit un texte directement"""
        # D'abord essayer avec les traductions .ts si disponibles
        if hasattr(self, "_ts_translations") and self._ts_translations:
            if text in self._ts_translations:
                return self._ts_translations[text]

        # Ensuite essayer avec QTranslator
        if hasattr(self.translator, "translate"):
            translated = self.translator.translate("EzQt_App", text)
            if translated:
                return translated
        return text

    def register_widget(self, widget, original_text: str):
        """Enregistre un widget pour retraduction automatique"""
        if widget not in self._translatable_widgets:
            self._translatable_widgets.append(widget)
        self._translatable_texts[widget] = original_text

    def unregister_widget(self, widget):
        """Désenregistre un widget de la retraduction automatique"""
        if widget in self._translatable_widgets:
            self._translatable_widgets.remove(widget)
        if widget in self._translatable_texts:
            del self._translatable_texts[widget]

    def set_translatable_text(self, widget, text: str):
        """Définit un texte traduit et l'enregistre pour retraduction automatique"""
        self.register_widget(widget, text)
        translated = self.translate(text)
        self._set_widget_text(widget, translated)

    def _set_widget_text(self, widget, text: str):
        """Définit le texte d'un widget selon son type"""
        if hasattr(widget, "setText"):
            widget.setText(text)
        elif hasattr(widget, "setTitle"):
            widget.setTitle(text)
        elif hasattr(widget, "setPlaceholderText"):
            widget.setPlaceholderText(text)
        elif hasattr(widget, "setToolTip"):
            widget.setToolTip(text)
        elif hasattr(widget, "setWindowTitle"):
            widget.setWindowTitle(text)
        else:
            pass

    def _retranslate_all_widgets(self):
        """Retraduit tous les widgets enregistrés"""
        for widget, original_text in self._translatable_texts.items():
            try:
                translated = self.translate(original_text)
                self._set_widget_text(widget, translated)
            except Exception as e:
                pass
        
        # Mettre à jour les widgets spéciaux qui nécessitent une logique particulière
        self._update_special_widgets()
    
    def _update_special_widgets(self):
        """Met à jour les widgets spéciaux qui nécessitent une logique particulière"""
        try:
            # Trouver tous les SettingsPanel dans l'application
            from PySide6.QtWidgets import QApplication
            app = QApplication.instance()
            if app:
                # Parcourir tous les widgets de l'application
                for widget in app.allWidgets():
                    # Vérifier si c'est un SettingsPanel
                    if hasattr(widget, 'update_theme_selector_items'):
                        try:
                            widget.update_theme_selector_items()
                        except Exception as e:
                            pass
        except Exception as e:
            pass

    def clear_registered_widgets(self):
        """Efface tous les widgets enregistrés"""
        self._translatable_widgets.clear()
        self._translatable_texts.clear()


# Instance globale (créée de manière lazy pour éviter les problèmes de contexte Qt)
_translation_manager_instance = None


def get_translation_manager():
    """Retourne l'instance globale du gestionnaire de traduction"""
    global _translation_manager_instance
    if _translation_manager_instance is None:
        _translation_manager_instance = TranslationManager()
    return _translation_manager_instance


# Alias pour compatibilité
translation_manager = get_translation_manager()



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# EZ APP
from .ez_app import *

# HEADER - KERNEL WIDGET
from .header import *

# MENU - KERNEL WIDGET
from .menu import *

# PAGE CONTAINER - KERNEL WIDGET
from .page_container import *

# SETTINGS PANEL - KERNEL WIDGET
from .settings_panel import *



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# MAINWINDOW
from .ui_main import Ui_MainWindow

# UI FUNCTIONS
from .ui_functions import UIFunctions

# APP FUNCTIONS
from .app_functions import Kernel

# APP SETTINGS
from .app_settings import Settings

# APP RESOURCES
from .app_components import Fonts, SizePolicy
from .app_resources import *
from .translation_manager import get_translation_manager, translation_manager
from .translation_helpers import tr, set_tr, register_tr, unregister_tr, change_language, get_available_languages, get_current_language


# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////
from typing import Dict, List

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import (
    Qt,
    QSize,
)
from PySide6.QtGui import (
    QCursor,
)
from PySide6.QtWidgets import (
    QWidget,
    QFrame,
    QVBoxLayout,
    QPushButton,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# /////////////////////////////////////////////////////////////////////////////////////////////
from ...kernel.app_components import *
from ...kernel.app_resources import *
from ...kernel.app_settings import Settings

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class Menu(QFrame):
    """
    This class is used to create a menu container.
    It contains a main menu frame and a toggle box.
    The main menu frame contains a top menu and a bottom menu.
    The toggle box contains a toggle button.
    The menu container is used to display the menu.
    """

    menus: Dict[str, QPushButton] = {}
    _buttons: List = []  # Type hint removed to avoid circular import
    _icons: List = []  # Type hint removed to avoid circular import

    # ///////////////////////////////////////////////////////////////

    def __init__(
        self, parent: QWidget = None, shrink_width: int = 60, extended_width: int = 240
    ) -> None:
        super(Menu, self).__init__(parent)

        # ///////////////////////////////////////////////////////////////
        # Store configuration
        self._shrink_width = shrink_width
        self._extended_width = extended_width

        self.setObjectName("menuContainer")
        self.setMinimumSize(QSize(self._shrink_width, 0))
        self.setMaximumSize(QSize(self._shrink_width, 16777215))
        self.setFrameShape(QFrame.NoFrame)
        self.setFrameShadow(QFrame.Raised)
        # //////
        self.VL_menuContainer = QVBoxLayout(self)
        self.VL_menuContainer.setSpacing(0)
        self.VL_menuContainer.setObjectName("VL_menuContainer")
        self.VL_menuContainer.setContentsMargins(0, 0, 0, 0)

        # ///////////////////////////////////////////////////////////////

        self.mainMenuFrame = QFrame(self)
        self.mainMenuFrame.setObjectName("mainMenuFrame")
        self.mainMenuFrame.setFrameShape(QFrame.NoFrame)
        self.mainMenuFrame.setFrameShadow(QFrame.Raised)
        #
        self.VL_menuContainer.addWidget(self.mainMenuFrame)
        # //////
        self.VL_mainMenuFrame = QVBoxLayout(self.mainMenuFrame)
        self.VL_mainMenuFrame.setSpacing(0)
        self.VL_mainMenuFrame.setObjectName("VL_mainMenuFrame")
        self.VL_mainMenuFrame.setContentsMargins(0, 0, 0, 0)

        # ToggleContainer for expand button
        # ///////////////////////////////////////////////////////////////

        self.toggleBox = QFrame(self.mainMenuFrame)
        self.toggleBox.setObjectName("toggleBox")
        self.toggleBox.setMaximumSize(QSize(16777215, 45))
        self.toggleBox.setFrameShape(QFrame.NoFrame)
        self.toggleBox.setFrameShadow(QFrame.Raised)
        #
        self.VL_mainMenuFrame.addWidget(self.toggleBox)
        # //////
        self.VL_toggleBox = QVBoxLayout(self.toggleBox)
        self.VL_toggleBox.setSpacing(0)
        self.VL_toggleBox.setObjectName("VL_toggleBox")
        self.VL_toggleBox.setContentsMargins(0, 0, 0, 0)

        # ///////////////////////////////////////////////////////////////

        # Lazy import to avoid circular imports
        from ...widgets.extended.menu_button import MenuButton
        from ...widgets.extended.theme_icon import ThemeIcon

        self.toggleButton = MenuButton(
            parent=self.toggleBox,
            icon=Icons.icon_menu,
            text="Hide",
            shrink_size=self._shrink_width,
            spacing=35,
            duration=Settings.Gui.TIME_ANIMATION,
        )
        self.toggleButton.setObjectName("toggleButton")
        self.toggleButton.setSizePolicy(SizePolicy.H_EXPANDING_V_FIXED)
        SizePolicy.H_EXPANDING_V_FIXED.setHeightForWidth(
            self.toggleButton.sizePolicy().hasHeightForWidth()
        )
        self.toggleButton.setMinimumSize(QSize(0, 45))
        self.toggleButton.setFont(Fonts.SEGOE_UI_10_REG)
        self.toggleButton.setCursor(QCursor(Qt.PointingHandCursor))
        self.toggleButton.setLayoutDirection(Qt.LeftToRight)
        # Don't set contents margins here - MenuButton handles its own positioning
        # self.toggleButton.setContentsMargins(20, 0, 0, 0)
        #
        icon_menu = ThemeIcon(Icons.icon_menu)
        self._buttons.append(self.toggleButton)
        self._icons.append(icon_menu)
        # Connect to the new toggle_state method
        self.toggleButton.clicked.connect(self.toggleButton.toggle_state)
        #
        self.VL_toggleBox.addWidget(self.toggleButton)

        # The Menu itself
        # ///////////////////////////////////////////////////////////////

        self.topMenu = QFrame(self.mainMenuFrame)
        self.topMenu.setObjectName("topMenu")
        self.topMenu.setFrameShape(QFrame.NoFrame)
        self.topMenu.setFrameShadow(QFrame.Raised)
        #
        self.VL_mainMenuFrame.addWidget(self.topMenu, 0, Qt.AlignTop)
        # //////
        self.VL_topMenu = QVBoxLayout(self.topMenu)
        self.VL_topMenu.setSpacing(0)
        self.VL_topMenu.setObjectName("VL_topMenu")
        self.VL_topMenu.setContentsMargins(0, 0, 0, 0)

        # ////// SYNC INITIAL STATE
        # Menu container starts shrinked (60px width), so sync toggle button state
        self._sync_initial_state()

    # ///////////////////////////////////////////////////////////////

    def _sync_initial_state(self):
        """Sync the initial state of all buttons with the menu container state."""
        # Menu container starts shrinked (configured width)
        # So all buttons should start in shrink state
        if hasattr(self, "toggleButton"):
            # Force toggle button to shrink state
            self.toggleButton.set_state(False)  # False = shrink state
            # Sync all existing menu buttons
            self.sync_all_menu_states(False)

    # ///////////////////////////////////////////////////////////////

    def add_menu(self, name: str, icon: str | Icons = None):
        # Lazy import to avoid circular imports
        from ...widgets.extended.menu_button import MenuButton
        from ...widgets.extended.theme_icon import ThemeIcon

        menu = MenuButton(
            parent=self.topMenu,
            icon=icon,
            text=name,
            shrink_size=self._shrink_width,
            spacing=35,
            duration=Settings.Gui.TIME_ANIMATION,
        )
        menu.setObjectName(f"menu_{name}")
        menu.setProperty("class", "inactive")
        menu.setSizePolicy(SizePolicy.H_EXPANDING_V_FIXED)
        SizePolicy.H_EXPANDING_V_FIXED.setHeightForWidth(
            menu.sizePolicy().hasHeightForWidth()
        )
        menu.setMinimumSize(QSize(0, 45))
        menu.setFont(Fonts.SEGOE_UI_10_REG)
        menu.setCursor(QCursor(Qt.PointingHandCursor))
        menu.setLayoutDirection(Qt.LeftToRight)
        # Don't set contents margins here - MenuButton handles its own positioning
        # menu.setContentsMargins(20, 0, 0, 0)
        #
        theme_icon = ThemeIcon(icon)
        self._buttons.append(menu)
        self._icons.append(theme_icon)
        # Connect to the toggle button to sync state
        self.toggleButton.stateChanged.connect(menu.set_state)
        #
        self.VL_topMenu.addWidget(menu)
        Menu.menus[name] = menu

        # //////
        return menu

    # ///////////////////////////////////////////////////////////////

    def update_all_theme_icons(self) -> None:
        """Update theme icons for all buttons."""
        for i, btn in enumerate(self._buttons):
            if hasattr(btn, "update_theme_icon") and self._icons[i]:
                # Use the dedicated method for theme icon updates
                btn.update_theme_icon(self._icons[i])

    def sync_all_menu_states(self, extended: bool) -> None:
        """Sync all menu buttons with the given state."""
        for btn in self._buttons:
            if btn != self.toggleButton:  # Don't sync the toggle button itself
                btn.set_state(extended)

    def get_menu_state(self) -> bool:
        """Get the current menu state (True for extended, False for shrink)."""
        if hasattr(self, "toggleButton"):
            return self.toggleButton.is_extended
        return False

    def get_shrink_width(self) -> int:
        """Get the configured shrink width."""
        return self._shrink_width

    def get_extended_width(self) -> int:
        """Get the configured extended width."""
        return self._extended_width



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////
import sys
import pkg_resources
from pathlib import Path
from typing import Dict, List
from colorama import Fore, Style
import yaml
import ruamel.yaml

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtGui import QFontDatabase
from PySide6.QtCore import (
    QSize,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////
from .app_settings import Settings
from ..helper import Helper

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////
APP_PATH = Path(getattr(sys, "_MEIPASS", Path(sys.argv[0]).resolve().parent))

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class Kernel:
    _yamlFile: None | Path = None

    # ///////////////////////////////////////////////////////////////

    @staticmethod
    def checkAssetsRequirements() -> None:
        # //////
        maker = Helper.Maker()  # Utilise APP_PATH par défaut
        maker.make_assets_binaries()
        res = maker.make_qrc()
        maker.make_rc_py() if res else maker.purge_rc_py()
        maker.make_app_resources_module()
        # //////
        print(Fore.LIGHTBLACK_EX + "..." + Style.RESET_ALL)

    # ///////////////////////////////////////////////////////////////

    @staticmethod
    def makeAppResourcesModule() -> None:
        # //////
        maker = Helper.Maker()  # Utilise APP_PATH par défaut
        maker.make_app_resources_module()

    # ///////////////////////////////////////////////////////////////

    @staticmethod
    def makeRequiredFiles(mkTheme: bool = True) -> None:
        # //////
        yaml_package = Kernel.getPackageResource("app.yaml")
        yaml_application = Helper.Maker(Path.cwd()).make_yaml(yaml_package)
        Kernel.yamlFile(yaml_application)
        # //////
        if mkTheme:
            theme_package = Kernel.getPackageResource("resources/themes/main_theme.qss")
            res = Helper.Maker(Path.cwd()).make_qss(theme_package)

        # //////
        # Copier les fichiers de traduction
        translations_package = Kernel.getPackageResource("resources/translations")
        translations_res = Helper.Maker(Path.cwd()).make_translations(translations_package)

        # //////
        if yaml_application or res is True or translations_res is True:
            print(Fore.LIGHTBLACK_EX + "..." + Style.RESET_ALL)

    # ///////////////////////////////////////////////////////////////

    @classmethod
    def yamlFile(cls, yamlFile: Path) -> None:
        # //////
        cls._yamlFile = yamlFile

    # ///////////////////////////////////////////////////////////////

    @classmethod
    def loadKernelConfig(cls, key: str) -> Dict[str, str | int]:
        # //////
        if not cls._yamlFile:
            cls._yamlFile = Kernel.getPackageResource("app.yaml")

        # //////
        with open(cls._yamlFile, "r", encoding='utf-8') as file:
            data = yaml.safe_load(file)
            # //////
            return data[key]

    # ///////////////////////////////////////////////////////////////

    @classmethod
    def writeYamlConfig(cls, keys: List[str], val: str | int | Dict[str, str]) -> None:
        # Protection contre la récursion
        if not hasattr(cls, '_writing_yaml'):
            cls._writing_yaml = False
        
        if cls._writing_yaml:
            return  # Éviter la récursion
        
        cls._writing_yaml = True
        
        try:
            yaml = ruamel.yaml.YAML()

            # //////
            with open(cls._yamlFile, "r", encoding='utf-8') as file:
                data = yaml.load(file)

            # //////
            d = data
            for key in keys[:-1]:
                d = d.setdefault(
                    key, ruamel.yaml.comments.CommentedMap()
                )  # Aller au niveau de la clé ou créer un nouveau dict
            d[keys[-1]] = val  # Mettre à jour la valeur de la dernière clé

            # //////
            with open(cls._yamlFile, "w", encoding='utf-8') as file:
                yaml.dump(data, file)
        finally:
            cls._writing_yaml = False

    # ///////////////////////////////////////////////////////////////

    @staticmethod
    def getPackageResource(resource_path: str) -> Path:
        resource = Path(pkg_resources.resource_filename("ezqt_app", resource_path))
        return resource

    # ///////////////////////////////////////////////////////////////

    @staticmethod
    def loadFontsResources(app: bool = False) -> None:
        # //////
        if not app:
            fonts = Kernel.getPackageResource("resources/fonts")
            source = "Package"
        else:
            fonts = APP_PATH / r"bin\fonts"
            source = "Application"

        # //////
        for font in fonts.iterdir():
            # //////
            if font.suffix == ".ttf":
                font_id = QFontDatabase.addApplicationFont(str(font))
                # //////
                if font_id == -1:
                    print(
                        Fore.LIGHTRED_EX
                        + f"! [AppKernel] | Failed to load from {source} : {font.stem}."
                        + Style.RESET_ALL
                    )
                # //////
                else:
                    print(
                        Fore.LIGHTBLUE_EX
                        + f"+ [AppKernel] | Font loaded from {source} : {font.stem}."
                        + Style.RESET_ALL
                    )

        # //////
        if not app:
            Kernel.loadFontsResources(app=True)
            # //////
            print(Fore.LIGHTBLACK_EX + "..." + Style.RESET_ALL)

    # ///////////////////////////////////////////////////////////////

    @staticmethod
    def loadAppSettings() -> Dict[str, str] | Dict[str, str]:
        # //////
        app_data = Kernel.loadKernelConfig("app")

        # //////
        Settings.App.NAME = app_data["name"]
        Settings.App.DESCRIPTION = app_data["description"]
        # //////
        Settings.App.ENABLE_CUSTOM_TITLE_BAR = True
        # //////
        Settings.App.APP_MIN_SIZE = QSize(
            app_data["app_min_width"], app_data["app_min_height"]
        )
        Settings.App.APP_WIDTH = app_data["app_width"]
        Settings.App.APP_HEIGHT = app_data["app_height"]

        # //////
        # Charger le thème depuis settings_panel s'il existe, sinon depuis app
        try:
            settings_panel = Kernel.loadKernelConfig("settings_panel")
            Settings.Gui.THEME = settings_panel.get("theme", {}).get("default", app_data["theme"])
        except KeyError:
            Settings.Gui.THEME = app_data["theme"]
            
        Settings.Gui.MENU_PANEL_EXTENDED_WIDTH = app_data["menu_panel_extended_width"]
        Settings.Gui.MENU_PANEL_SHRINKED_WIDTH = app_data["menu_panel_shrinked_width"]
        Settings.Gui.SETTINGS_PANEL_WIDTH = app_data["settings_panel_width"]
        Settings.Gui.TIME_ANIMATION = app_data["time_animation"]

        # //////
        print(
            Fore.LIGHTBLUE_EX
            + f"+ [AppKernel] | Loaded Application settings."
            + Style.RESET_ALL
        )
        # //////
        print(
            Fore.LIGHTBLACK_EX
            + "   ┌───────────────────────────────────────────────┐"
            + Style.RESET_ALL
        )
        for key, val in app_data.items():
            print(
                Fore.LIGHTBLACK_EX
                + f"   |- {key}: "
                + Fore.LIGHTWHITE_EX
                + f"{val}"
                + Style.RESET_ALL
            )
        print(
            Fore.LIGHTBLACK_EX
            + "   └───────────────────────────────────────────────┘"
            + Style.RESET_ALL
        )

        # //////
        print(Fore.LIGHTBLACK_EX + "..." + Style.RESET_ALL)



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////
import sys
import subprocess
import shutil
from pathlib import Path

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from colorama import Fore, Style

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////
APP_PATH = Path(getattr(sys, "_MEIPASS", Path(sys.argv[0]).resolve().parent))

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class Helper:
    def __init__(self) -> None:
        pass

    # MAKER
    # ///////////////////////////////////////////////////////////////
    class Maker(object):
        def __init__(self, base_path=None):
            self.base_path = base_path or APP_PATH
            self._bin = self.base_path / "bin"
            self._modules = self.base_path / "modules"
            self._qrc_file = ""
            self._resources_module_file = ""

        # ///////////////////////////////////////////////////////////////

        def make_assets_binaries(self) -> None:
            trace = False
            paths_to_make = [
                self._bin,
                self._bin / "fonts",
                self._bin / "images",
                self._bin / "icons",
                self._bin / "themes",
                self._bin / "config",
                self._bin / "translations",
                self._modules,
            ]
            for path in paths_to_make:
                if not path.exists():
                    path.mkdir(parents=True, exist_ok=True)
                    trace = True
            if trace:
                print(
                    Fore.LIGHTBLACK_EX
                    + f"~ [Helper] | Generated assets binaries."
                    + Style.RESET_ALL
                )

        # ///////////////////////////////////////////////////////////////

        def make_yaml(self, yaml_package: Path) -> Path | None:
            yaml_application = self.base_path / r"bin\config\app.yaml"
            if not yaml_application.exists():
                shutil.copy(yaml_package, yaml_application)
                print(
                    Fore.LIGHTBLACK_EX
                    + "~ [Helper] | Generated required yaml file."
                    + Style.RESET_ALL
                )
            return yaml_application

        # ///////////////////////////////////////////////////////////////

        def make_qss(self, theme_package) -> bool:
            theme_application = self.base_path / r"bin\themes\main_theme.qss"
            if not theme_application.exists():
                shutil.copy(theme_package, theme_application)
                print(
                    Fore.LIGHTBLACK_EX
                    + "~ [Helper] | Generated optional qss file."
                    + Style.RESET_ALL
                )
                return True
            return False

        # ///////////////////////////////////////////////////////////////

        def make_translations(self, translations_package: Path) -> bool:
            """Copie les fichiers de traduction du package vers le projet utilisateur"""
            translations_application = self.base_path / r"bin\translations"
            if not translations_application.exists():
                translations_application.mkdir(parents=True, exist_ok=True)
            
            # Copier tous les fichiers .qm et .ts du package
            copied_files = []
            if translations_package.exists():
                # Copier les fichiers .qm
                for qm_file in translations_package.glob("*.qm"):
                    dest_file = translations_application / qm_file.name
                    if not dest_file.exists():
                        shutil.copy(qm_file, dest_file)
                        copied_files.append(qm_file.name)
                
                # Copier les fichiers .ts
                for ts_file in translations_package.glob("*.ts"):
                    dest_file = translations_application / ts_file.name
                    if not dest_file.exists():
                        shutil.copy(ts_file, dest_file)
                        copied_files.append(ts_file.name)
                
                if copied_files:
                    print(
                        Fore.LIGHTBLACK_EX
                        + f"~ [Helper] | Generated translation files: {', '.join(copied_files)}"
                        + Style.RESET_ALL
                    )
                    return True
            
            return False

        # ///////////////////////////////////////////////////////////////

        def make_qrc(self) -> bool:
            def _add_qresource(directory: Path, prefix: str) -> int:
                valid_extensions = {".png", ".jpg", ".jpeg", ".ico", ".svg"}
                entries = [
                    f
                    for f in directory.iterdir()
                    if f.suffix.lower() in valid_extensions
                ]
                if entries:
                    self._qrc_file += f'  <qresource prefix="{prefix}">\n'
                    for f in entries:
                        self._qrc_file += f"    <file>{prefix}/{f.name}</file>\n"
                    self._qrc_file += "  </qresource>\n"
                    return 1
                return 0
            count = 0
            self._qrc_file = "<RCC>\n"
            count += _add_qresource(self._bin / "images", "images")
            count += _add_qresource(self._bin / "icons", "icons")
            self._qrc_file += "</RCC>\n"
            with open(self._bin / "resources.qrc", mode="w") as f:
                f.write(self._qrc_file)
            print(
                Fore.LIGHTBLACK_EX
                + "~ [Helper] | Generated QRC file from bin folder content."
                + Style.RESET_ALL
            )
            return count != 0

        # ///////////////////////////////////////////////////////////////

        def make_rc_py(self) -> None:
            qrc_path = self._bin / "resources.qrc"
            pyrc_path = self._modules / "resources_rc.py"
            result = subprocess.run(
                ["pyside6-rcc", str(qrc_path), "-o", str(pyrc_path)],
                capture_output=True,
                text=True,
            )
            if result.returncode == 0:
                print(
                    Fore.LIGHTBLACK_EX
                    + "~ [Helper] | Converted QRC file to PY."
                    + Style.RESET_ALL
                )
            else:
                print(
                    Fore.LIGHTRED_EX + f"! [Helper] | {result.stderr}" + Style.RESET_ALL
                )

        # ///////////////////////////////////////////////////////////////

        def purge_rc_py(self) -> None:
            rc_py = self._modules / "resources_rc.py"
            rc_py.unlink() if rc_py.exists() else None

        # ///////////////////////////////////////////////////////////////

        def make_app_resources_module(self) -> None:
            def _add_cls_attr(directory: Path, attr: str) -> int:
                valid_extensions = {".png", ".jpg", ".jpeg", ".ico", ".svg"}
                entries = [
                    f
                    for f in directory.iterdir()
                    if f.suffix.lower() in valid_extensions
                ]
                self._resources_module_file += (
                    f"class App{attr.capitalize()}({attr.capitalize()}):\n"
                )
                if entries:
                    for f in entries:
                        self._resources_module_file += f"    {f.stem.replace('-', '_')} = ':/{attr}/{attr}/{f.name}'\n"
                    self._resources_module_file += "\n\n"
                    return 1
                else:
                    self._resources_module_file += "    pass\n"
                    self._resources_module_file += "\n\n"
                    return 0
            count = 0
            self._resources_module_file = """
from ezqt_app.kernel.app_resources import Icons, Images
\n\n\n"""
            count += _add_cls_attr(self._bin / "images", "images")
            count += _add_cls_attr(self._bin / "icons", "icons")
            if count != 0:
                self._resources_module_file = (
                    "from .resources_rc import *\n" + self._resources_module_file
                )
            with open(self._modules / "app_resources.py", mode="w") as f:
                f.write(self._resources_module_file)
            print(
                Fore.LIGHTBLACK_EX
                + "~ [Helper] | Generated app_resources.py file. Ready for use."
                + Style.RESET_ALL
            )

        # ///////////////////////////////////////////////////////////////

        def make_generic_main(self, mainTxtTemplate: Path) -> None:
            main_txt = self.base_path / r"main.txt"
            main_py = self.base_path / r"main.py"
            if not main_py.exists():
                shutil.copy(mainTxtTemplate, main_txt)
                main_txt.rename(main_py)
            print(
                Fore.LIGHTBLACK_EX
                + "~ [Helper] | Generated example main.py file."
                + Style.RESET_ALL
            )



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////
import sys
import platform
from pathlib import Path
from colorama import Fore, Style

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import *
from PySide6.QtGui import *
from PySide6.QtWidgets import *

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////
from .kernel import *

# Import specific widgets to avoid circular imports
from .widgets.core.ez_app import EzApplication

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////
os_name = platform.system()
widgets = None

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////
APP_PATH = Path(getattr(sys, "_MEIPASS", Path(sys.argv[0]).resolve().parent))
# //////
_dev = True if not hasattr(sys, "frozen") else False

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class EzQt_App(QMainWindow):
    def __init__(
        self,
        themeFileName: str = None,
    ) -> None:
        QMainWindow.__init__(self)

        # ==> KERNEL LOADER
        # ///////////////////////////////////////////////////////////////.
        Kernel.loadFontsResources()
        Kernel.loadAppSettings()
        # ==> LOAD TRANSLATIONS
        # ///////////////////////////////////////////////////////////////.
        from .kernel.translation_manager import get_translation_manager

        # Charger la langue depuis les paramètres
        try:
            settings_panel = Kernel.loadKernelConfig("settings_panel")
            language = settings_panel.get("language", {}).get("default", "English")
            translation_manager = get_translation_manager()
            translation_manager.load_language(language)
        except KeyError:
            translation_manager = get_translation_manager()
            translation_manager.load_language("English")

        # ==> INITIALIZE COMPONENTS
        # ///////////////////////////////////////////////////////////////.
        Fonts.initFonts()
        SizePolicy.initSizePolicy()

        # ==> SET AS GLOBAL WIDGETS
        # ///////////////////////////////////////////////////////////////
        self.ui = Ui_MainWindow()
        self.ui.setupUi(self)
        global widgets
        widgets = self.ui

        # ==> USE CUSTOM TITLE BAR | "True" for Windows
        # ///////////////////////////////////////////////////////////////
        Settings.App.ENABLE_CUSTOM_TITLE_BAR = True if os_name == "Windows" else False

        # ==> APP DATA
        # ///////////////////////////////////////////////////////////////
        self.setWindowTitle(Settings.App.NAME)
        (
            self.setAppIcon(Images.logo_placeholder, yShrink=0)
            if Settings.Gui.THEME == "dark"
            else self.setAppIcon(Images.logo_placeholder, yShrink=0)
        )

        # ==> TOGGLE MENU
        # ///////////////////////////////////////////////////////////////
        widgets.menuContainer.toggleButton.clicked.connect(
            lambda: UIFunctions.toggleMenuPanel(self, True)
        )

        # ==> TOGGLE SETTINGS
        # ///////////////////////////////////////////////////////////////
        widgets.headerContainer.settingsTopBtn.clicked.connect(
            lambda: UIFunctions.toggleSettingsPanel(self, True)
        )

        # SET UI DEFINITIONS
        # ///////////////////////////////////////////////////////////////
        UIFunctions.uiDefinitions(self)

        # SET THEME
        # ///////////////////////////////////////////////////////////////
        self._themeFileName = themeFileName
        UIFunctions.theme(self, self._themeFileName)
        # //////
        # Charger le thème depuis settings_panel s'il existe, sinon depuis app
        try:
            settings_panel = Kernel.loadKernelConfig("settings_panel")
            _theme = settings_panel.get("theme", {}).get(
                "default", Kernel.loadKernelConfig("app")["theme"]
            )
            # Forcer la conversion en minuscules
            _theme = _theme.lower()
        except KeyError:
            _theme = Kernel.loadKernelConfig("app")["theme"]
            # Forcer la conversion en minuscules
            _theme = _theme.lower()

        # Mettre à jour Settings.Gui.THEME avec la valeur en minuscules
        Settings.Gui.THEME = _theme

        theme_toggle = self.ui.settingsPanel.get_theme_toggle_button()
        if theme_toggle and hasattr(theme_toggle, "initialize_selector"):
            # Convertir la valeur de thème en ID
            theme_id = 0 if _theme == "light" else 1  # 0 = Light, 1 = Dark
            theme_toggle.initialize_selector(theme_id)
        self.ui.headerContainer.update_all_theme_icons()
        self.ui.menuContainer.update_all_theme_icons()
        # //////
        if theme_toggle:
            # Connecter le signal valueChanged au lieu de clicked pour la nouvelle version
            if hasattr(theme_toggle, "valueChanged"):
                theme_toggle.valueChanged.connect(self.setAppTheme)
            elif hasattr(theme_toggle, "clicked"):
                theme_toggle.clicked.connect(self.setAppTheme)

        # ==> REGISTER ALL WIDGETS FOR TRANSLATION
        # ///////////////////////////////////////////////////////////////
        self._register_all_widgets_for_translation()

    # SET APP THEME
    # ///////////////////////////////////////////////////////////////
    def setAppTheme(self) -> None:
        theme_toggle = self.ui.settingsPanel.get_theme_toggle_button()
        if theme_toggle:
            # Gérer les deux cas : valueChanged (chaîne) et clicked (pas de paramètre)
            if hasattr(theme_toggle, "value_id"):
                # Utiliser value_id pour obtenir l'ID actuel
                theme_id = theme_toggle.value_id
                # Convertir l'ID en valeur de thème
                theme = "light" if theme_id == 0 else "dark"
                Settings.Gui.THEME = theme
                # Sauvegarder directement dans settings_panel.theme.default
                Kernel.writeYamlConfig(
                    keys=["settings_panel", "theme", "default"], val=theme
                )
                # //////
                QTimer.singleShot(100, self.updateUI)

    # UPDATE UI
    # ///////////////////////////////////////////////////////////////
    def updateUI(self) -> None:
        theme_toggle = self.ui.settingsPanel.get_theme_toggle_button()
        if theme_toggle and hasattr(theme_toggle, "get_value_option"):
            # La nouvelle version d'OptionSelector gère automatiquement le positionnement
            # Pas besoin de move_selector manuel
            pass

        # //////
        UIFunctions.theme(self, self._themeFileName)
        # //////
        EzApplication.instance().themeChanged.emit()
        self.ui.headerContainer.update_all_theme_icons()
        self.ui.menuContainer.update_all_theme_icons()
        self.ui.settingsPanel.update_all_theme_icons()

        # //////
        QApplication.processEvents()

    # SET APP ICON
    # ///////////////////////////////////////////////////////////////
    def setAppIcon(
        self, icon: str | QPixmap, yShrink: int = 0, yOffset: int = 0
    ) -> None:
        return self.ui.headerContainer.set_app_logo(
            logo=icon, y_shrink=yShrink, y_offset=yOffset
        )

    # ADD MENU & PAGE
    # ///////////////////////////////////////////////////////////////
    def addMenu(self, name: str, icon: str) -> QWidget:
        page = self.ui.pagesContainer.add_page(name)
        # //////
        menu = self.ui.menuContainer.add_menu(name, icon)
        menu.setProperty("page", page)
        if len(self.ui.menuContainer.menus) == 1:
            menu.setProperty("class", "active")
        # //////
        menu.clicked.connect(
            lambda: widgets.pagesContainer.stackedWidget.setCurrentWidget(page)
        )
        menu.clicked.connect(self.switchMenu)

        # //////
        return page

    # MENU SWITCH
    # ///////////////////////////////////////////////////////////////
    def switchMenu(self) -> None:
        # GET BUTTON CLICKED
        sender = self.sender()
        senderName = sender.objectName()

        # SHOW HOME PAGE
        for btnName, btnWidget in self.ui.menuContainer.menus.items():
            if senderName == f"menu_{btnName}":
                UIFunctions.deselectMenu(self, senderName)
                UIFunctions.selectMenu(self, senderName)

    # RESIZE EVENTS
    # ///////////////////////////////////////////////////////////////
    def resizeEvent(self, event) -> None:
        # Update Size Grips
        UIFunctions.resize_grips(self)

    # REGISTER ALL WIDGETS FOR TRANSLATION
    # ///////////////////////////////////////////////////////////////
    def _register_all_widgets_for_translation(self) -> None:
        """Enregistre automatiquement tous les widgets avec du texte pour la traduction."""
        try:
            # Import sécurisé des fonctions de traduction
            try:
                from .kernel.translation_helpers import set_tr, tr as translate_text
            except ImportError as import_error:
                print(f"Warning: Could not import translation helpers: {import_error}")
                return

            registered_count = 0
            registered_widgets = set()  # Pour éviter les doublons

            def register_widget_recursive(widget):
                """Fonction récursive pour enregistrer tous les widgets."""
                nonlocal registered_count

                try:
                    # Éviter les widgets déjà enregistrés
                    if widget in registered_widgets:
                        return

                    # Vérifier si le widget a du texte
                    if hasattr(widget, "text") and callable(
                        getattr(widget, "text", None)
                    ):
                        try:
                            text = widget.text().strip()
                            # Éviter les widgets avec du texte technique, valeurs numériques, ou trop courts
                            if (
                                text
                                and not text.isdigit()
                                and len(text) > 1
                                and not text.startswith("_")
                                and not text.startswith("menu_")
                                and not text.startswith("btn_")
                                and not text.startswith("setting")
                            ):

                                # Enregistrer le widget pour la traduction
                                set_tr(widget, text)
                                registered_widgets.add(widget)
                                registered_count += 1
                        except Exception as text_error:
                            # Ignorer les erreurs de lecture de texte
                            pass

                    # Vérifier les tooltips
                    if hasattr(widget, "toolTip") and callable(
                        getattr(widget, "toolTip", None)
                    ):
                        try:
                            tooltip = widget.toolTip().strip()
                            if (
                                tooltip
                                and not tooltip.isdigit()
                                and len(tooltip) > 1
                                and not tooltip.startswith("_")
                            ):
                                # Pour les tooltips, on peut utiliser setToolTip avec tr()
                                widget.setToolTip(translate_text(tooltip))
                        except Exception as tooltip_error:
                            # Ignorer les erreurs de tooltip
                            pass

                    # Vérifier les placeholders
                    if hasattr(widget, "placeholderText") and callable(
                        getattr(widget, "placeholderText", None)
                    ):
                        try:
                            placeholder = widget.placeholderText().strip()
                            if (
                                placeholder
                                and not placeholder.isdigit()
                                and len(placeholder) > 1
                                and not placeholder.startswith("_")
                            ):
                                widget.setPlaceholderText(translate_text(placeholder))
                        except Exception as placeholder_error:
                            # Ignorer les erreurs de placeholder
                            pass

                    # Parcourir tous les enfants
                    try:
                        for child in widget.findChildren(QWidget):
                            register_widget_recursive(child)
                    except Exception as children_error:
                        # Ignorer les erreurs de recherche d'enfants
                        pass

                except Exception as widget_error:
                    # Ignorer les erreurs individuelles de widgets
                    pass

            # Commencer par la fenêtre principale
            register_widget_recursive(self)

            # Enregistrer manuellement les widgets spécifiques avec du texte fixe
            self._register_specific_widgets_for_translation()

            print(
                Fore.LIGHTBLUE_EX
                + f"+ [AppKernel] | {registered_count} widgets registered for translation."
                + Style.RESET_ALL
            )

        except Exception as e:
            print(f"Warning: Could not register widgets for translation: {e}")

    def _register_specific_widgets_for_translation(self) -> None:
        """Enregistre manuellement les widgets spécifiques avec du texte fixe."""
        try:
            from .kernel.translation_helpers import set_tr

            # Widgets dans ui_main.py avec du texte fixe
            if hasattr(self.ui, "creditsLabel"):
                set_tr(self.ui.creditsLabel, "Made with ❤️ by EzQt_App")

            if hasattr(self.ui, "version"):
                set_tr(self.ui.version, "v1.0.0")

            # Widgets dans settings_panel avec du texte fixe
            if hasattr(self.ui, "settingsPanel"):
                settings_panel = self.ui.settingsPanel

                # Enregistrer les widgets de paramètres créés dynamiquement
                for widget in getattr(settings_panel, "_widgets", []):
                    if hasattr(widget, "label") and widget.label:
                        try:
                            text = widget.label.text()
                            if text and len(text) > 1:
                                set_tr(widget.label, text)
                        except:
                            pass

                # Enregistrer le label de thème
                if hasattr(settings_panel, "themeLabel"):
                    set_tr(settings_panel.themeLabel, "Theme")

                # Enregistrer les options du sélecteur de thème
                if hasattr(settings_panel, "themeToggleButton"):
                    theme_button = settings_panel.themeToggleButton
                    try:
                        # La nouvelle version d'OptionSelector gère automatiquement les traductions
                        # Pas besoin d'enregistrer manuellement les items
                        pass
                    except:
                        pass

            # Widgets dans menu avec du texte fixe
            if hasattr(self.ui, "menuContainer"):
                menu_container = self.ui.menuContainer

                # Enregistrer les boutons de menu créés dynamiquement
                for button in getattr(menu_container, "_buttons", []):
                    if hasattr(button, "text_label") and button.text_label:
                        try:
                            text = button.text_label.text()
                            if text and len(text) > 1:
                                set_tr(button.text_label, text)
                        except:
                            pass

            # Widgets dans header avec du texte fixe
            if hasattr(self.ui, "headerContainer"):
                header_container = self.ui.headerContainer

                # Enregistrer les labels du header
                if hasattr(header_container, "headerAppName"):
                    try:
                        text = header_container.headerAppName.text()
                        if text and len(text) > 1:
                            set_tr(header_container.headerAppName, text)
                    except:
                        pass

                if hasattr(header_container, "headerAppDescription"):
                    try:
                        text = header_container.headerAppDescription.text()
                        if text and len(text) > 1:
                            set_tr(header_container.headerAppDescription, text)
                    except:
                        pass

            # Widgets de ezqt_widgets avec du texte
            # Note: Ces widgets gèrent généralement leurs propres traductions
            # mais on peut enregistrer leurs textes pour la retraduction automatique

            # ToggleSwitch widgets (dans setting_widgets)
            # OptionSelector widgets (dans settings_panel)
            # Ces widgets sont déjà gérés par l'enregistrement automatique

        except Exception as e:
            # Ignorer les erreurs pour cette fonction
            pass

    # MOUSE CLICK EVENTS
    # ///////////////////////////////////////////////////////////////
    def mousePressEvent(self, event) -> None:
        # SET DRAG POS WINDOW
        self.dragPos = event.globalPosition().toPoint()

        # //////
        if _dev:
            # PRINT OBJECT NAME
            # //////
            child_widget = self.childAt(event.position().toPoint())
            if child_widget:
                child_name = child_widget.objectName()
                print(child_name)

            # PRINT MOUSE EVENTS
            # //////
            elif event.buttons() == Qt.LeftButton:
                print(f"Mouse click: LEFT CLICK")
            elif event.buttons() == Qt.RightButton:
                print("Mouse click: RIGHT CLICK")



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////

# IMPORT / GUI AND MODULES AND WIDGETS
# /////////////////////////////////////////////////////////////////////////////////////////////
from .base_resources import *

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class Images:
    pass
    logo_placeholder = ":/images/images/logo-placeholder.jpg"


class Icons:
    pass
    cil_3d = ":/icons/icons/cil-3d.png"
    cil_4k = ":/icons/icons/cil-4k.png"
    cil_account_logout = ":/icons/icons/cil-account-logout.png"
    cil_action_redo = ":/icons/icons/cil-action-redo.png"
    cil_action_undo = ":/icons/icons/cil-action-undo.png"
    cil_airplane_mode_off = ":/icons/icons/cil-airplane-mode-off.png"
    cil_airplane_mode = ":/icons/icons/cil-airplane-mode.png"
    cil_alarm = ":/icons/icons/cil-alarm.png"
    cil_align_center = ":/icons/icons/cil-align-center.png"
    cil_align_left = ":/icons/icons/cil-align-left.png"
    cil_align_right = ":/icons/icons/cil-align-right.png"
    cil_arrow_bottom_2 = ":/icons/icons/cil-arrow-bottom-2.png"
    cil_arrow_bottom = ":/icons/icons/cil-arrow-bottom.png"
    cil_arrow_circle_bottom = ":/icons/icons/cil-arrow-circle-bottom.png"
    cil_arrow_circle_left = ":/icons/icons/cil-arrow-circle-left.png"
    cil_arrow_circle_right = ":/icons/icons/cil-arrow-circle-right.png"
    cil_arrow_circle_top = ":/icons/icons/cil-arrow-circle-top.png"
    cil_arrow_left = ":/icons/icons/cil-arrow-left.png"
    cil_arrow_right = ":/icons/icons/cil-arrow-right.png"
    cil_arrow_top = ":/icons/icons/cil-arrow-top.png"
    cil_at = ":/icons/icons/cil-at.png"
    cil_av_timer = ":/icons/icons/cil-av-timer.png"
    cil_ban = ":/icons/icons/cil-ban.png"
    cil_battery_0 = ":/icons/icons/cil-battery-0.png"
    cil_battery_3 = ":/icons/icons/cil-battery-3.png"
    cil_battery_5 = ":/icons/icons/cil-battery-5.png"
    cil_battery_alert = ":/icons/icons/cil-battery-alert.png"
    cil_battery_slash = ":/icons/icons/cil-battery-slash.png"
    cil_bell = ":/icons/icons/cil-bell.png"
    cil_bluetooth = ":/icons/icons/cil-bluetooth.png"
    cil_briefcase = ":/icons/icons/cil-briefcase.png"
    cil_browser = ":/icons/icons/cil-browser.png"
    cil_calendar_check = ":/icons/icons/cil-calendar-check.png"
    cil_camera_roll = ":/icons/icons/cil-camera-roll.png"
    cil_camera = ":/icons/icons/cil-camera.png"
    cil_caret_bottom = ":/icons/icons/cil-caret-bottom.png"
    cil_caret_left = ":/icons/icons/cil-caret-left.png"
    cil_caret_right = ":/icons/icons/cil-caret-right.png"
    cil_caret_top = ":/icons/icons/cil-caret-top.png"
    cil_cart = ":/icons/icons/cil-cart.png"
    cil_cast = ":/icons/icons/cil-cast.png"
    cil_chart_line = ":/icons/icons/cil-chart-line.png"
    cil_chart_pie = ":/icons/icons/cil-chart-pie.png"
    cil_chart = ":/icons/icons/cil-chart.png"
    cil_chat_bubble = ":/icons/icons/cil-chat-bubble.png"
    cil_check_alt = ":/icons/icons/cil-check-alt.png"
    cil_check_circle = ":/icons/icons/cil-check-circle.png"
    cil_check = ":/icons/icons/cil-check.png"
    cil_chevron_bottom = ":/icons/icons/cil-chevron-bottom.png"
    cil_chevron_circle_down_alt = ":/icons/icons/cil-chevron-circle-down-alt.png"
    cil_chevron_circle_left_alt = ":/icons/icons/cil-chevron-circle-left-alt.png"
    cil_chevron_circle_right_alt = ":/icons/icons/cil-chevron-circle-right-alt.png"
    cil_chevron_circle_up_alt = ":/icons/icons/cil-chevron-circle-up-alt.png"
    cil_chevron_double_down = ":/icons/icons/cil-chevron-double-down.png"
    cil_chevron_double_left = ":/icons/icons/cil-chevron-double-left.png"
    cil_chevron_double_right = ":/icons/icons/cil-chevron-double-right.png"
    cil_chevron_double_up_alt = ":/icons/icons/cil-chevron-double-up-alt.png"
    cil_chevron_double_up = ":/icons/icons/cil-chevron-double-up.png"
    cil_chevron_left = ":/icons/icons/cil-chevron-left.png"
    cil_chevron_right = ":/icons/icons/cil-chevron-right.png"
    cil_chevron_top = ":/icons/icons/cil-chevron-top.png"
    cil_circle = ":/icons/icons/cil-circle.png"
    cil_clipboard = ":/icons/icons/cil-clipboard.png"
    cil_clock = ":/icons/icons/cil-clock.png"
    cil_clone = ":/icons/icons/cil-clone.png"
    cil_closed_captioning = ":/icons/icons/cil-closed-captioning.png"
    cil_cloud_download = ":/icons/icons/cil-cloud-download.png"
    cil_cloud_upload = ":/icons/icons/cil-cloud-upload.png"
    cil_cloudy = ":/icons/icons/cil-cloudy.png"
    cil_code = ":/icons/icons/cil-code.png"
    cil_coffee = ":/icons/icons/cil-coffee.png"
    cil_comment_bubble = ":/icons/icons/cil-comment-bubble.png"
    cil_comment_square = ":/icons/icons/cil-comment-square.png"
    cil_credit_card = ":/icons/icons/cil-credit-card.png"
    cil_cursor_move = ":/icons/icons/cil-cursor-move.png"
    cil_cursor = ":/icons/icons/cil-cursor.png"
    cil_cut = ":/icons/icons/cil-cut.png"
    cil_data_transfer_down = ":/icons/icons/cil-data-transfer-down.png"
    cil_data_transfer_up = ":/icons/icons/cil-data-transfer-up.png"
    cil_deaf = ":/icons/icons/cil-deaf.png"
    cil_description = ":/icons/icons/cil-description.png"
    cil_devices = ":/icons/icons/cil-devices.png"
    cil_dialpad = ":/icons/icons/cil-dialpad.png"
    cil_dog = ":/icons/icons/cil-dog.png"
    cil_door = ":/icons/icons/cil-door.png"
    cil_double_quote_sans_left = ":/icons/icons/cil-double-quote-sans-left.png"
    cil_double_quote_sans_right = ":/icons/icons/cil-double-quote-sans-right.png"
    cil_drop = ":/icons/icons/cil-drop.png"
    cil_envelope_closed = ":/icons/icons/cil-envelope-closed.png"
    cil_envelope_letter = ":/icons/icons/cil-envelope-letter.png"
    cil_envelope_open = ":/icons/icons/cil-envelope-open.png"
    cil_equalizer = ":/icons/icons/cil-equalizer.png"
    cil_ethernet = ":/icons/icons/cil-ethernet.png"
    cil_exit_to_app = ":/icons/icons/cil-exit-to-app.png"
    cil_expand_down = ":/icons/icons/cil-expand-down.png"
    cil_expand_left = ":/icons/icons/cil-expand-left.png"
    cil_expand_right = ":/icons/icons/cil-expand-right.png"
    cil_expand_up = ":/icons/icons/cil-expand-up.png"
    cil_exposure = ":/icons/icons/cil-exposure.png"
    cil_external_link = ":/icons/icons/cil-external-link.png"
    cil_face_dead = ":/icons/icons/cil-face-dead.png"
    cil_featured_playlist = ":/icons/icons/cil-featured-playlist.png"
    cil_file = ":/icons/icons/cil-file.png"
    cil_find_in_page = ":/icons/icons/cil-find-in-page.png"
    cil_fingerprint = ":/icons/icons/cil-fingerprint.png"
    cil_fire = ":/icons/icons/cil-fire.png"
    cil_flip_to_back = ":/icons/icons/cil-flip-to-back.png"
    cil_folder_open = ":/icons/icons/cil-folder-open.png"
    cil_folder = ":/icons/icons/cil-folder.png"
    cil_frown = ":/icons/icons/cil-frown.png"
    cil_gamepad = ":/icons/icons/cil-gamepad.png"
    cil_hand_point_down = ":/icons/icons/cil-hand-point-down.png"
    cil_hand_point_left = ":/icons/icons/cil-hand-point-left.png"
    cil_hand_point_right = ":/icons/icons/cil-hand-point-right.png"
    cil_hand_point_up = ":/icons/icons/cil-hand-point-up.png"
    cil_hd = ":/icons/icons/cil-hd.png"
    cil_hdr = ":/icons/icons/cil-hdr.png"
    cil_headphones = ":/icons/icons/cil-headphones.png"
    cil_heart = ":/icons/icons/cil-heart.png"
    cil_highligt = ":/icons/icons/cil-highligt.png"
    cil_history = ":/icons/icons/cil-history.png"
    cil_home = ":/icons/icons/cil-home.png"
    cil_house = ":/icons/icons/cil-house.png"
    cil_image_plus = ":/icons/icons/cil-image-plus.png"
    cil_image1 = ":/icons/icons/cil-image1.png"
    cil_infinity = ":/icons/icons/cil-infinity.png"
    cil_input_power = ":/icons/icons/cil-input-power.png"
    cil_input = ":/icons/icons/cil-input.png"
    cil_justify_center = ":/icons/icons/cil-justify-center.png"
    cil_justify_left = ":/icons/icons/cil-justify-left.png"
    cil_justify_right = ":/icons/icons/cil-justify-right.png"
    cil_keyboard = ":/icons/icons/cil-keyboard.png"
    cil_laptop = ":/icons/icons/cil-laptop.png"
    cil_layers = ":/icons/icons/cil-layers.png"
    cil_level_down = ":/icons/icons/cil-level-down.png"
    cil_level_up = ":/icons/icons/cil-level-up.png"
    cil_library_add = ":/icons/icons/cil-library-add.png"
    cil_library = ":/icons/icons/cil-library.png"
    cil_lightbulb = ":/icons/icons/cil-lightbulb.png"
    cil_link_alt = ":/icons/icons/cil-link-alt.png"
    cil_link_broken = ":/icons/icons/cil-link-broken.png"
    cil_link = ":/icons/icons/cil-link.png"
    cil_location_pin = ":/icons/icons/cil-location-pin.png"
    cil_lock_locked = ":/icons/icons/cil-lock-locked.png"
    cil_lock_unlocked = ":/icons/icons/cil-lock-unlocked.png"
    cil_loop_1 = ":/icons/icons/cil-loop-1.png"
    cil_loop_circular = ":/icons/icons/cil-loop-circular.png"
    cil_loop = ":/icons/icons/cil-loop.png"
    cil_low_vision = ":/icons/icons/cil-low-vision.png"
    cil_magnifying_glass = ":/icons/icons/cil-magnifying-glass.png"
    cil_map = ":/icons/icons/cil-map.png"
    cil_media_eject = ":/icons/icons/cil-media-eject.png"
    cil_media_pause = ":/icons/icons/cil-media-pause.png"
    cil_media_play = ":/icons/icons/cil-media-play.png"
    cil_media_skip_backward = ":/icons/icons/cil-media-skip-backward.png"
    cil_media_skip_forward = ":/icons/icons/cil-media-skip-forward.png"
    cil_media_step_backward = ":/icons/icons/cil-media-step-backward.png"
    cil_media_step_forward = ":/icons/icons/cil-media-step-forward.png"
    cil_media_stop = ":/icons/icons/cil-media-stop.png"
    cil_medical_cross = ":/icons/icons/cil-medical-cross.png"
    cil_meh = ":/icons/icons/cil-meh.png"
    cil_menu = ":/icons/icons/cil-menu.png"
    cil_microphone = ":/icons/icons/cil-microphone.png"
    cil_minus = ":/icons/icons/cil-minus.png"
    cil_mobile_landscape = ":/icons/icons/cil-mobile-landscape.png"
    cil_mobile = ":/icons/icons/cil-mobile.png"
    cil_mood_bad = ":/icons/icons/cil-mood-bad.png"
    cil_mood_good = ":/icons/icons/cil-mood-good.png"
    cil_mood_very_bad = ":/icons/icons/cil-mood-very-bad.png"
    cil_mood_very_good = ":/icons/icons/cil-mood-very-good.png"
    cil_moon = ":/icons/icons/cil-moon.png"
    cil_mouse = ":/icons/icons/cil-mouse.png"
    cil_move = ":/icons/icons/cil-move.png"
    cil_movie = ":/icons/icons/cil-movie.png"
    cil_mug_tea = ":/icons/icons/cil-mug-tea.png"
    cil_mug = ":/icons/icons/cil-mug.png"
    cil_notes = ":/icons/icons/cil-notes.png"
    cil_options_horizontal = ":/icons/icons/cil-options-horizontal.png"
    cil_options = ":/icons/icons/cil-options.png"
    cil_paint_bucket = ":/icons/icons/cil-paint-bucket.png"
    cil_paper_plane = ":/icons/icons/cil-paper-plane.png"
    cil_paperclip = ":/icons/icons/cil-paperclip.png"
    cil_paragraph = ":/icons/icons/cil-paragraph.png"
    cil_pen_alt = ":/icons/icons/cil-pen-alt.png"
    cil_pencil = ":/icons/icons/cil-pencil.png"
    cil_people = ":/icons/icons/cil-people.png"
    cil_phone = ":/icons/icons/cil-phone.png"
    cil_pin = ":/icons/icons/cil-pin.png"
    cil_plus = ":/icons/icons/cil-plus.png"
    cil_power_standby = ":/icons/icons/cil-power-standby.png"
    cil_print = ":/icons/icons/cil-print.png"
    cil_rectangle = ":/icons/icons/cil-rectangle.png"
    cil_reload = ":/icons/icons/cil-reload.png"
    cil_remove = ":/icons/icons/cil-remove.png"
    cil_rss = ":/icons/icons/cil-rss.png"
    cil_satelite = ":/icons/icons/cil-satelite.png"
    cil_save = ":/icons/icons/cil-save.png"
    cil_screen_desktop = ":/icons/icons/cil-screen-desktop.png"
    cil_screen_smartphone = ":/icons/icons/cil-screen-smartphone.png"
    cil_settings = ":/icons/icons/cil-settings.png"
    cil_share_boxed = ":/icons/icons/cil-share-boxed.png"
    cil_share = ":/icons/icons/cil-share.png"
    cil_signal_cellular_0 = ":/icons/icons/cil-signal-cellular-0.png"
    cil_signal_cellular_3 = ":/icons/icons/cil-signal-cellular-3.png"
    cil_size_grip = ":/icons/icons/cil-size-grip.png"
    cil_smile = ":/icons/icons/cil-smile.png"
    cil_speaker = ":/icons/icons/cil-speaker.png"
    cil_speech = ":/icons/icons/cil-speech.png"
    cil_speedometer = ":/icons/icons/cil-speedometer.png"
    cil_star = ":/icons/icons/cil-star.png"
    cil_tags = ":/icons/icons/cil-tags.png"
    cil_task = ":/icons/icons/cil-task.png"
    cil_terminal = ":/icons/icons/cil-terminal.png"
    cil_text_size = ":/icons/icons/cil-text-size.png"
    cil_text_square = ":/icons/icons/cil-text-square.png"
    cil_text = ":/icons/icons/cil-text.png"
    cil_thumb_down = ":/icons/icons/cil-thumb-down.png"
    cil_thumb_up = ":/icons/icons/cil-thumb-up.png"
    cil_transfer = ":/icons/icons/cil-transfer.png"
    cil_trash = ":/icons/icons/cil-trash.png"
    cil_triangle = ":/icons/icons/cil-triangle.png"
    cil_truck = ":/icons/icons/cil-truck.png"
    cil_user_female = ":/icons/icons/cil-user-female.png"
    cil_user_follow = ":/icons/icons/cil-user-follow.png"
    cil_user_unfollow = ":/icons/icons/cil-user-unfollow.png"
    cil_user = ":/icons/icons/cil-user.png"
    cil_vertical_align_bottom = ":/icons/icons/cil-vertical-align-bottom.png"
    cil_view_column = ":/icons/icons/cil-view-column.png"
    cil_view_module = ":/icons/icons/cil-view-module.png"
    cil_view_quilt = ":/icons/icons/cil-view-quilt.png"
    cil_view_stream = ":/icons/icons/cil-view-stream.png"
    cil_voice_over_record = ":/icons/icons/cil-voice-over-record.png"
    cil_volume_high = ":/icons/icons/cil-volume-high.png"
    cil_volume_low = ":/icons/icons/cil-volume-low.png"
    cil_volume_off = ":/icons/icons/cil-volume-off.png"
    cil_wallet = ":/icons/icons/cil-wallet.png"
    cil_watch = ":/icons/icons/cil-watch.png"
    cil_wifi_signal_0 = ":/icons/icons/cil-wifi-signal-0.png"
    cil_wifi_signal_1 = ":/icons/icons/cil-wifi-signal-1.png"
    cil_wifi_signal_2 = ":/icons/icons/cil-wifi-signal-2.png"
    cil_wifi_signal_4 = ":/icons/icons/cil-wifi-signal-4.png"
    cil_wifi_signal_off = ":/icons/icons/cil-wifi-signal-off.png"
    cil_window_maximize = ":/icons/icons/cil-window-maximize.png"
    cil_window_minimize = ":/icons/icons/cil-window-minimize.png"
    cil_window_restore = ":/icons/icons/cil-window-restore.png"
    cil_wrap_text = ":/icons/icons/cil-wrap-text.png"
    cil_x_circle = ":/icons/icons/cil-x-circle.png"
    cil_x = ":/icons/icons/cil-x.png"
    cil_zoom_in = ":/icons/icons/cil-zoom-in.png"
    cil_zoom_out = ":/icons/icons/cil-zoom-out.png"
    icon_close = ":/icons/icons/icon_close.png"
    icon_maximize = ":/icons/icons/icon_maximize.png"
    icon_menu = ":/icons/icons/icon_menu.png"
    icon_minimize = ":/icons/icons/icon_minimize.png"
    icon_restore = ":/icons/icons/icon_restore.png"
    icon_settings = ":/icons/icons/icon_settings.png"
    material_circled_C = ":/icons/icons/material-circled-C.png"
    material_circled_K = ":/icons/icons/material-circled-K.png"
    material_copy_24 = ":/icons/icons/material-copy-24.png"
    material_electricity = ":/icons/icons/material-electricity.png"
    material_play_blue = ":/icons/icons/material-play-blue.png"
    material_stop_red = ":/icons/icons/material-stop-red.png"



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////
from typing import List, Optional, Any
from PySide6.QtCore import Signal, Qt, QSize
from PySide6.QtWidgets import (
    QWidget,
    QVBoxLayout,
    QHBoxLayout,
    QLabel,
    QCheckBox,
    QComboBox,
    QSlider,
    QLineEdit,
    QFrame,
    QPushButton,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# /////////////////////////////////////////////////////////////////////////////////////////////
from ...kernel.app_components import *
from ...kernel.app_settings import Settings
from ezqt_widgets import ToggleSwitch

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class BaseSettingWidget(QWidget):
    """Classe de base pour tous les widgets de paramètres."""

    def __init__(self, label: str, description: str = ""):
        super().__init__()
        self._label = label
        self._description = description
        self._key = None
        self.setObjectName("BaseSettingWidget")

    def set_key(self, key: str):
        """Définit la clé du paramètre."""
        self._key = key
    



class SettingToggle(BaseSettingWidget):
    """Widget pour les paramètres toggle (on/off)."""

    def __init__(self, label: str, description: str = "", default: bool = False):
        super().__init__(label, description)
        self._value = default
        self.setObjectName("SettingToggle")
        self.setProperty("type", "SettingToggle")
        self._setup_ui()

    def _setup_ui(self):
        """Configure l'interface utilisateur."""
        # Layout principal
        layout = QVBoxLayout(self)
        layout.setSpacing(4)
        layout.setContentsMargins(0, 0, 0, 0)

        # Container pour le label et le toggle
        control_layout = QHBoxLayout()
        control_layout.setSpacing(8)
        control_layout.setContentsMargins(0, 0, 0, 0)

        # Label principal
        self.label = QLabel(self._label)
        self.label.setObjectName("settingLabel")
        self.label.setWordWrap(True)
        control_layout.addWidget(self.label, 1)  # Stretch

        # Toggle (ToggleSwitch moderne)
        # Get animation setting from app settings
        animation_enabled = getattr(Settings.Gui, "TIME_ANIMATION", 400) > 0

        self.toggle = ToggleSwitch(animation=animation_enabled)
        self.toggle.setObjectName("settingToggle")
        self.toggle.checked = self._value
        self.toggle.toggled.connect(self._on_toggled)
        control_layout.addWidget(self.toggle, 0)  # No stretch

        layout.addLayout(control_layout)

        # Description (si présente)
        if self._description:
            self.description_label = QLabel(self.tr(self._description))
            self.description_label.setObjectName("settingDescription")
            self.description_label.setWordWrap(True)
            layout.addWidget(self.description_label)

    def _on_toggled(self, checked: bool):
        """Appelé quand le toggle change."""
        self._value = checked
        self.valueChanged.emit(self._key, checked)

    @property
    def value(self) -> bool:
        """Récupère la valeur actuelle."""
        return self._value

    @value.setter
    def value(self, val: bool):
        """Définit la valeur."""
        self._value = val
        self.toggle.checked = val

    def get_value(self) -> bool:
        """Récupère la valeur actuelle."""
        return self._value

    def set_value(self, val: bool):
        """Définit la valeur."""
        self._value = val
        self.toggle.checked = val

    valueChanged = Signal(str, bool)


class SettingSelect(BaseSettingWidget):
    """Widget pour les paramètres de sélection."""

    def __init__(
        self,
        label: str,
        description: str = "",
        options: list = None,
        default: str = None,
    ):
        super().__init__(label, description)
        self._options = options or []
        self._value = default or (options[0] if options else "")
        self.setObjectName("SettingSelect")
        self.setProperty("type", "SettingSelect")
        self._setup_ui()

    def _setup_ui(self):
        """Configure l'interface utilisateur."""
        # Layout principal
        layout = QVBoxLayout(self)
        layout.setSpacing(4)
        layout.setContentsMargins(0, 0, 0, 0)

        # Label principal
        self.label = QLabel(self._label)
        self.label.setObjectName("settingLabel")
        self.label.setWordWrap(True)
        layout.addWidget(self.label)

        # ComboBox
        self.combo = QComboBox()
        self.combo.setObjectName("settingComboBox")
        self.combo.addItems(self._options)
        if self._value in self._options:
            self.combo.setCurrentText(self._value)
        self.combo.currentTextChanged.connect(self._on_text_changed)
        layout.addWidget(self.combo)

        # Description (si présente)
        if self._description:
            self.description_label = QLabel(self.tr(self._description))
            self.description_label.setObjectName("settingDescription")
            self.description_label.setWordWrap(True)
            layout.addWidget(self.description_label)

    def _on_text_changed(self, text: str):
        """Appelé quand la sélection change."""
        self._value = text
        self.valueChanged.emit(self._key, text)

    @property
    def value(self) -> str:
        """Récupère la valeur actuelle."""
        return self._value

    @value.setter
    def value(self, val: str):
        """Définit la valeur."""
        self._value = val
        if val in self._options:
            self.combo.setCurrentText(val)

    def get_value(self) -> str:
        """Récupère la valeur actuelle."""
        return self._value

    def set_value(self, val: str):
        """Définit la valeur."""
        self._value = val
        if val in self._options:
            self.combo.setCurrentText(val)

    valueChanged = Signal(str, str)


class SettingSlider(BaseSettingWidget):
    """Widget pour les paramètres numériques avec slider."""

    def __init__(
        self,
        label: str,
        description: str = "",
        min_val: int = 0,
        max_val: int = 100,
        default: int = 50,
        unit: str = "",
    ):
        super().__init__(label, description)
        self._min_val = min_val
        self._max_val = max_val
        self._value = default
        self._unit = unit
        self.setObjectName("SettingSlider")
        self.setProperty("type", "SettingSlider")
        self._setup_ui()

    def _setup_ui(self):
        """Configure l'interface utilisateur."""
        # Layout principal
        layout = QVBoxLayout(self)
        layout.setSpacing(4)
        layout.setContentsMargins(0, 0, 0, 0)

        # Container pour le label et la valeur
        header_layout = QHBoxLayout()
        header_layout.setSpacing(8)
        header_layout.setContentsMargins(0, 0, 0, 0)

        # Label principal
        self.label = QLabel(self._label)
        self.label.setObjectName("settingLabel")
        self.label.setWordWrap(True)
        header_layout.addWidget(self.label, 1)  # Stretch

        # Label de valeur
        self.value_label = QLabel(f"{self._value}{self._unit}")
        self.value_label.setObjectName("settingValueLabel")
        header_layout.addWidget(self.value_label, 0)  # No stretch

        layout.addLayout(header_layout)

        # Slider
        self.slider = QSlider(Qt.Horizontal)
        self.slider.setObjectName("settingSlider")
        self.slider.setMinimum(self._min_val)
        self.slider.setMaximum(self._max_val)
        self.slider.setValue(self._value)
        self.slider.valueChanged.connect(self._on_value_changed)
        layout.addWidget(self.slider)

        # Description (si présente)
        if self._description:
            self.description_label = QLabel(self.tr(self._description))
            self.description_label.setObjectName("settingDescription")
            self.description_label.setWordWrap(True)
            layout.addWidget(self.description_label)

    def _on_value_changed(self, value: int):
        """Appelé quand la valeur du slider change."""
        self._value = value
        self.value_label.setText(f"{value}{self._unit}")
        self.valueChanged.emit(self._key, value)

    @property
    def value(self) -> int:
        """Récupère la valeur actuelle."""
        return self._value

    @value.setter
    def value(self, val: int):
        """Définit la valeur."""
        self._value = val
        self.slider.setValue(val)
        self.value_label.setText(f"{val}{self._unit}")

    def get_value(self) -> int:
        """Récupère la valeur actuelle."""
        return self._value

    def set_value(self, val: int):
        """Définit la valeur."""
        self._value = val
        self.slider.setValue(val)
        self.value_label.setText(f"{val}{self._unit}")

    valueChanged = Signal(str, int)


class SettingText(BaseSettingWidget):
    """Widget pour les paramètres texte."""

    def __init__(self, label: str, description: str = "", default: str = ""):
        super().__init__(label, description)
        self._value = default
        self.setObjectName("SettingText")
        self.setProperty("type", "SettingText")
        self._setup_ui()

    def _setup_ui(self):
        """Configure l'interface utilisateur."""
        # Layout principal
        layout = QVBoxLayout(self)
        layout.setSpacing(4)
        layout.setContentsMargins(0, 0, 0, 0)

        # Label principal
        self.label = QLabel(self._label)
        self.label.setObjectName("settingLabel")
        self.label.setWordWrap(True)
        layout.addWidget(self.label)

        # Champ texte
        self.text_edit = QLineEdit()
        self.text_edit.setObjectName("settingTextEdit")
        self.text_edit.setText(self._value)
        self.text_edit.textChanged.connect(self._on_text_changed)
        layout.addWidget(self.text_edit)

        # Description (si présente)
        if self._description:
            self.description_label = QLabel(self.tr(self._description))
            self.description_label.setObjectName("settingDescription")
            self.description_label.setWordWrap(True)
            layout.addWidget(self.description_label)

    def _on_text_changed(self, text: str):
        """Appelé quand le texte change."""
        self._value = text
        self.valueChanged.emit(self._key, text)

    @property
    def value(self) -> str:
        """Récupère la valeur actuelle."""
        return self._value

    @value.setter
    def value(self, val: str):
        """Définit la valeur."""
        self._value = val
        self.text_edit.setText(val)

    def get_value(self) -> str:
        """Récupère la valeur actuelle."""
        return self._value

    def set_value(self, val: str):
        """Définit la valeur."""
        self._value = val
        self.text_edit.setText(val)

    valueChanged = Signal(str, str)


class SettingCheckbox(BaseSettingWidget):
    """Widget pour les paramètres checkbox (on/off)."""

    def __init__(self, label: str, description: str = "", default: bool = False):
        super().__init__(label, description)
        self._value = default
        self.setObjectName("SettingCheckbox")
        self.setProperty("type", "SettingCheckbox")
        self._setup_ui()

    def _setup_ui(self):
        """Configure l'interface utilisateur."""
        # Layout principal
        layout = QVBoxLayout(self)
        layout.setSpacing(4)
        layout.setContentsMargins(0, 0, 0, 0)

        # Container pour le label et la checkbox
        control_layout = QHBoxLayout()
        control_layout.setSpacing(8)
        control_layout.setContentsMargins(0, 0, 0, 0)

        # Label principal
        self.label = QLabel(self._label)
        self.label.setObjectName("settingLabel")
        self.label.setWordWrap(True)
        control_layout.addWidget(self.label, 1)  # Stretch

        # Checkbox (ToggleSwitch moderne)
        # Get animation setting from app settings
        animation_enabled = getattr(Settings.Gui, "TIME_ANIMATION", 400) > 0

        self.checkbox = ToggleSwitch(animation=animation_enabled)
        self.checkbox.setObjectName("settingCheckbox")
        self.checkbox.checked = self._value
        self.checkbox.toggled.connect(self._on_toggled)
        control_layout.addWidget(self.checkbox, 0)  # No stretch

        layout.addLayout(control_layout)

        # Description (si présente)
        if self._description:
            self.description_label = QLabel(self.tr(self._description))
            self.description_label.setObjectName("settingDescription")
            self.description_label.setWordWrap(True)
            layout.addWidget(self.description_label)

    def _on_toggled(self, checked: bool):
        """Appelé quand la checkbox change."""
        self._value = checked
        self.valueChanged.emit(self._key, checked)

    @property
    def value(self) -> bool:
        """Récupère la valeur actuelle."""
        return self._value

    @value.setter
    def value(self, val: bool):
        """Définit la valeur."""
        self._value = val
        self.checkbox.checked = val

    def get_value(self) -> bool:
        """Récupère la valeur actuelle."""
        return self._value

    def set_value(self, val: bool):
        """Définit la valeur."""
        self._value = val
        self.checkbox.checked = val

    valueChanged = Signal(str, bool)



# -*- coding: utf-8 -*-
# /////////////////////////////////////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# /////////////////////////////////////////////////////////////////////////////////////////////

# IMPORT BASE
# /////////////////////////////////////////////////////////////////////////////////////////////

# IMPORT SPECS
# /////////////////////////////////////////////////////////////////////////////////////////////
from PySide6.QtCore import (
    Qt,
    QSize,
    QMetaObject,
)
from PySide6.QtWidgets import (
    QMainWindow,
    QWidget,
    QFrame,
    QVBoxLayout,
    QHBoxLayout,
    QLabel,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# /////////////////////////////////////////////////////////////////////////////////////////////
from .app_components import *
from .app_resources import *
from ..widgets.core import *
from ..kernel.app_settings import Settings

## ==> GLOBALS
# /////////////////////////////////////////////////////////////////////////////////////////////

## ==> VARIABLES
# /////////////////////////////////////////////////////////////////////////////////////////////

## ==> CLASSES
# /////////////////////////////////////////////////////////////////////////////////////////////


class Ui_MainWindow(object):
    def __init__(self) -> None:
        pass

    # ///////////////////////////////////////////////////////////////

    def setupUi(self, MainWindow: QMainWindow) -> None:
        if not MainWindow.objectName():
            MainWindow.setObjectName("MainWindow")
        MainWindow.resize(1280, 720)
        MainWindow.setMinimumSize(QSize(940, 560))

        # ///////////////////////////////////////////////////////////////

        self.styleSheet = QWidget(MainWindow)
        self.styleSheet.setObjectName("styleSheet")
        self.styleSheet.setFont(Fonts.SEGOE_UI_10_REG)
        # //////
        self.appMargins = QVBoxLayout(self.styleSheet)
        self.appMargins.setSpacing(0)
        self.appMargins.setObjectName("appMargins")
        self.appMargins.setContentsMargins(10, 10, 10, 10)

        # ///////////////////////////////////////////////////////////////

        self.bgApp = QFrame(self.styleSheet)
        self.bgApp.setObjectName("bgApp")
        self.bgApp.setStyleSheet("")
        self.bgApp.setFrameShape(QFrame.NoFrame)
        self.bgApp.setFrameShadow(QFrame.Raised)
        #
        self.appMargins.addWidget(self.bgApp)
        # //////
        self.appLayout = QVBoxLayout(self.bgApp)
        self.appLayout.setSpacing(0)
        self.appLayout.setObjectName("appLayout")
        self.appLayout.setContentsMargins(0, 0, 0, 0)

        # ///////////////////////////////////////////////////////////////
        # BALISE HEADER
        # ///////////////////////////////////////////////////////////////

        self.headerContainer = Header(parent=self.bgApp)
        #
        self.appLayout.addWidget(self.headerContainer)

        # ///////////////////////////////////////////////////////////////
        # END HEADER
        # ///////////////////////////////////////////////////////////////

        self.contentBox = QFrame(self.bgApp)
        self.contentBox.setObjectName("contentBox")
        self.contentBox.setFrameShape(QFrame.NoFrame)
        self.contentBox.setFrameShadow(QFrame.Raised)
        #
        self.appLayout.addWidget(self.contentBox)
        # //////
        self.HL_contentBox = QHBoxLayout(self.contentBox)
        self.HL_contentBox.setSpacing(0)
        self.HL_contentBox.setObjectName("HL_contentBox")
        self.HL_contentBox.setContentsMargins(0, 0, 0, 0)

        # ///////////////////////////////////////////////////////////////
        # BALISE MENU
        # ///////////////////////////////////////////////////////////////

        self.menuContainer = Menu(
            parent=self.contentBox,
            shrink_width=Settings.Gui.MENU_PANEL_SHRINKED_WIDTH,
            extended_width=Settings.Gui.MENU_PANEL_EXTENDED_WIDTH,
        )
        #
        self.HL_contentBox.addWidget(self.menuContainer)

        # ///////////////////////////////////////////////////////////////
        # END MENU
        # ///////////////////////////////////////////////////////////////

        self.contentBottom = QFrame(self.contentBox)
        self.contentBottom.setObjectName("contentBottom")
        self.contentBottom.setFrameShape(QFrame.NoFrame)
        self.contentBottom.setFrameShadow(QFrame.Raised)
        #
        self.HL_contentBox.addWidget(self.contentBottom)
        # //////
        self.VL_contentBottom = QVBoxLayout(self.contentBottom)
        self.VL_contentBottom.setSpacing(0)
        self.VL_contentBottom.setObjectName("VL_contentBottom")
        self.VL_contentBottom.setContentsMargins(0, 0, 0, 0)

        self.content = QFrame(self.contentBottom)
        self.content.setObjectName("content")
        self.content.setFrameShape(QFrame.NoFrame)
        self.content.setFrameShadow(QFrame.Raised)
        #
        self.VL_contentBottom.addWidget(self.content)
        # //////
        self.HL_content = QHBoxLayout(self.content)
        self.HL_content.setSpacing(0)
        self.HL_content.setObjectName("HL_content")
        self.HL_content.setContentsMargins(0, 0, 0, 0)

        # ///////////////////////////////////////////////////////////////
        # BALISE PAGE CONTAINER
        # ///////////////////////////////////////////////////////////////

        self.pagesContainer = PageContainer(self.contentBottom)
        #
        self.HL_content.addWidget(self.pagesContainer)

        # ///////////////////////////////////////////////////////////////
        # END PAGE CONTAINER
        # ///////////////////////////////////////////////////////////////

        # ///////////////////////////////////////////////////////////////
        # BALISE SETTINGS PANEL
        # ///////////////////////////////////////////////////////////////

        self.settingsPanel = SettingsPanel(
            parent=self.content,
            width=Settings.Gui.SETTINGS_PANEL_WIDTH,
        )
        #
        self.HL_content.addWidget(self.settingsPanel)

        # ///////////////////////////////////////////////////////////////
        # END SETTINGS PANEL
        # ///////////////////////////////////////////////////////////////

        self.bottomBar = QFrame(self.contentBottom)
        self.bottomBar.setObjectName("bottomBar")
        self.bottomBar.setMinimumSize(QSize(0, 22))
        self.bottomBar.setMaximumSize(QSize(16777215, 22))
        self.bottomBar.setFrameShape(QFrame.NoFrame)
        self.bottomBar.setFrameShadow(QFrame.Raised)
        #
        self.VL_contentBottom.addWidget(self.bottomBar)
        # //////
        self.HL_bottomBar = QHBoxLayout(self.bottomBar)
        self.HL_bottomBar.setSpacing(0)
        self.HL_bottomBar.setObjectName("HL_bottomBar")
        self.HL_bottomBar.setContentsMargins(0, 0, 0, 0)

        # //////
        self.creditsLabel = QLabel(self.bottomBar)
        self.creditsLabel.setObjectName("creditsLabel")
        self.creditsLabel.setMaximumSize(QSize(16777215, 16))
        self.creditsLabel.setFont(Fonts.SEGOE_UI_10_REG)
        self.creditsLabel.setAlignment(Qt.AlignLeading | Qt.AlignLeft | Qt.AlignVCenter)
        #
        self.HL_bottomBar.addWidget(self.creditsLabel)

        # //////
        self.version = QLabel(self.bottomBar)
        self.version.setObjectName("version")
        self.version.setAlignment(Qt.AlignRight | Qt.AlignTrailing | Qt.AlignVCenter)
        #
        self.HL_bottomBar.addWidget(self.version)

        # //////
        self.appSizeGrip = QFrame(self.bottomBar)
        self.appSizeGrip.setObjectName("appSizeGrip")
        self.appSizeGrip.setMinimumSize(QSize(20, 0))
        self.appSizeGrip.setMaximumSize(QSize(20, 16777215))
        self.appSizeGrip.setFrameShape(QFrame.NoFrame)
        self.appSizeGrip.setFrameShadow(QFrame.Raised)
        #
        self.HL_bottomBar.addWidget(self.appSizeGrip)

        # ///////////////////////////////////////////////////////////////
        # ///////////////////////////////////////////////////////////////

        MainWindow.setCentralWidget(self.styleSheet)
        QMetaObject.connectSlotsByName(MainWindow)



# -*- coding: utf-8 -*-
"""
Configuration des traductions pour EzQt_App
"""

# Langues supportées
SUPPORTED_LANGUAGES = {
    "en": {
        "name": "English",
        "native_name": "English",
        "file": "ezqt_app_en.ts"
    },
    "fr": {
        "name": "Français", 
        "native_name": "Français",
        "file": "ezqt_app_fr.ts"
    },
    "es": {
        "name": "Español",
        "native_name": "Español", 
        "file": "ezqt_app_es.ts"
    },
    "de": {
        "name": "Deutsch",
        "native_name": "Deutsch",
        "file": "ezqt_app_de.ts"
    }
}

# Langue par défaut
DEFAULT_LANGUAGE = "en"

# Dossier des traductions
TRANSLATIONS_DIR = "resources/translations"



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////
#
# BY: WANDERSON M.PIMENTA
# PROJECT MADE WITH: Qt Designer and PySide6
# V: 1.0.0
#
# This project can be used freely for all uses, as long as they maintain the
# respective credits only in the Python scripts, any information in the visual
# interface (GUI) can be modified without any implication.
#
# There are limitations on Qt licenses if you want to use your products
# commercially, I recommend reading them on the official website:
# https://doc.qt.io/qtforpython/licenses.html
#
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////
import sys
from pathlib import Path
from typing import Any

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import (
    Qt,
    QEvent,
    QTimer,
    QPropertyAnimation,
    QEasingCurve,
)
from PySide6.QtGui import (
    QIcon,
    QColor,
)
from PySide6.QtWidgets import (
    QWidget,
    QToolButton,
    QSizeGrip,
    QGraphicsDropShadowEffect,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////
from .app_settings import Settings
from .app_functions import Kernel
from ..widgets.custom_grips.custom_grips import CustomGrip

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////
GLOBAL_STATE = False
GLOBAL_TITLE_BAR = True

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////
APP_PATH = Path(getattr(sys, "_MEIPASS", Path(sys.argv[0]).resolve().parent))

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class UIFunctions:
    def __init__(self) -> None:
        from .ui_main import Ui_MainWindow

        self.ui: Ui_MainWindow

    # MAXIMIZE/RESTORE
    # ///////////////////////////////////////////////////////////////
    def maximize_restore(self) -> None:
        global GLOBAL_STATE
        status = GLOBAL_STATE
        if status == False:
            self.showMaximized()
            GLOBAL_STATE = True
            self.ui.appMargins.setContentsMargins(0, 0, 0, 0)
            self.ui.headerContainer.maximizeRestoreAppBtn.setToolTip("Restore")
            self.ui.headerContainer.maximizeRestoreAppBtn.setIcon(
                QIcon(":/icons/icons/icon_restore.png")
            )
            self.ui.appSizeGrip.hide()
            self.left_grip.hide()
            self.right_grip.hide()
            self.top_grip.hide()
            self.bottom_grip.hide()
        else:
            GLOBAL_STATE = False
            self.showNormal()
            self.resize(self.width() + 1, self.height() + 1)
            self.ui.appMargins.setContentsMargins(10, 10, 10, 10)
            self.ui.headerContainer.maximizeRestoreAppBtn.setToolTip("Maximize")
            self.ui.headerContainer.maximizeRestoreAppBtn.setIcon(
                QIcon(":/icons/icons/icon_maximize.png")
            )
            self.ui.appSizeGrip.show()
            self.left_grip.show()
            self.right_grip.show()
            self.top_grip.show()
            self.bottom_grip.show()

    # RETURN STATUS
    # ///////////////////////////////////////////////////////////////
    def returnStatus(self) -> Any | bool:
        return GLOBAL_STATE

    # SET STATUS
    # ///////////////////////////////////////////////////////////////
    def setStatus(self, status) -> None:
        global GLOBAL_STATE
        GLOBAL_STATE = status

    # TOGGLE MENU PANEL
    # ///////////////////////////////////////////////////////////////
    def toggleMenuPanel(self, enable) -> None:
        if enable:
            # GET WIDTH
            width = self.ui.menuContainer.width()
            maxExtend = self.ui.menuContainer.get_extended_width()
            standard = self.ui.menuContainer.get_shrink_width()

            # SET MAX WIDTH
            if width == self.ui.menuContainer.get_shrink_width():
                widthExtended = maxExtend
            else:
                widthExtended = standard

            # ANIMATION
            self.menu_animation = QPropertyAnimation(
                self.ui.menuContainer, b"minimumWidth"
            )
            self.menu_animation.setDuration(Settings.Gui.TIME_ANIMATION)
            self.menu_animation.setStartValue(width)
            self.menu_animation.setEndValue(widthExtended)
            self.menu_animation.setEasingCurve(QEasingCurve.InOutQuart)
            self.menu_animation.start()

    # TOGGLE SETTINGS PANEL
    # ///////////////////////////////////////////////////////////////
    def toggleSettingsPanel(self, enable) -> None:
        if enable:
            # GET WIDTH
            width = self.ui.settingsPanel.width()
            maxExtend = Settings.Gui.SETTINGS_PANEL_WIDTH
            standard = 0

            # SET MAX WIDTH
            if width == 0:
                widthExtended = maxExtend
            else:
                widthExtended = standard

            # ANIMATION
            self.settings_animation = QPropertyAnimation(
                self.ui.settingsPanel, b"minimumWidth"
            )
            self.settings_animation.setDuration(Settings.Gui.TIME_ANIMATION)
            self.settings_animation.setStartValue(width)
            self.settings_animation.setEndValue(widthExtended)
            self.settings_animation.setEasingCurve(QEasingCurve.InOutQuart)
            self.settings_animation.start()

            # Synchronisation du toggle avec le thème courant
            current_theme = Settings.Gui.THEME
            theme_toggle = self.ui.settingsPanel.get_theme_toggle_button()
            if theme_toggle and hasattr(theme_toggle, 'initialize_selector'):
                theme_toggle.initialize_selector(current_theme)

    # SELECT/DESELECT MENU
    # ///////////////////////////////////////////////////////////////
    # START SELECTION
    def selectMenu(self, widget) -> None:
        for w in self.ui.menuContainer.topMenu.findChildren(QToolButton):
            if w.objectName() == widget and isinstance(w, QToolButton):
                w.setProperty("class", "active")
                UIFunctions.refreshStyle(w)

    # RESET SELECTION
    def deselectMenu(self, widget) -> None:
        for w in self.ui.menuContainer.topMenu.findChildren(QToolButton):
            if w.objectName() != widget and isinstance(w, QToolButton):
                w.setProperty("class", "inactive")
                UIFunctions.refreshStyle(w)

    # REFRESH STYLE
    @staticmethod
    def refreshStyle(w: QWidget) -> None:
        w.style().unpolish(w)
        w.style().polish(w)

    # IMPORT THEMES FILES QSS/CSS
    # ///////////////////////////////////////////////////////////////
    def theme(self, customThemeFile: str = None) -> None:
        _style = ""
        # Utiliser Settings.Gui.THEME qui a été mis à jour par loadAppSettings
        _theme = Settings.Gui.THEME
        _colors = Kernel.loadKernelConfig("theme_palette")[_theme]

        # Main Theme
        # ///////////////////////////////////////////////////////////////
        if customThemeFile:
            main_qss = APP_PATH / rf"bin\themes\{customThemeFile}"
        else:
            main_qss = Kernel.getPackageResource("resources/themes/main_theme.qss")

        main_style = open(main_qss, "r").read()
        # //////
        for key, color in _colors.items():
            main_style = main_style.replace(key, color)
        # //////
        _style += f"{main_style}\n"

        # QtStrap
        # ///////////////////////////////////////////////////////////////
        qtstrap_qss = Kernel.getPackageResource("resources/themes/qtstrap.qss")
        qtstrap_style = open(qtstrap_qss, "r").read()
        # //////
        _style += f"{qtstrap_style}\n"

        # //////
        self.ui.styleSheet.setStyleSheet(_style)

    # START - GUI DEFINITIONS
    # ///////////////////////////////////////////////////////////////
    def uiDefinitions(self) -> None:
        def doubleClickMaximizeRestore(event) -> None:
            # IF DOUBLE CLICK CHANGE STATUS
            if event.type() == QEvent.MouseButtonDblClick:
                QTimer.singleShot(250, lambda: UIFunctions.maximize_restore(self))

        self.ui.headerContainer.mouseDoubleClickEvent = doubleClickMaximizeRestore

        # //////
        if Settings.App.NAME:
            self.ui.headerContainer.set_app_name(Settings.App.NAME)
        # //////
        if Settings.App.DESCRIPTION:
            self.ui.headerContainer.set_app_description(Settings.App.DESCRIPTION)
        # //////
        if Settings.App.APP_WIDTH and Settings.App.APP_HEIGHT:
            self.resize(Settings.App.APP_WIDTH, Settings.App.APP_HEIGHT)
        # //////
        if Settings.App.APP_MIN_SIZE:
            self.setMinimumSize(Settings.App.APP_MIN_SIZE)
        # //////
        if Settings.App.ENABLE_CUSTOM_TITLE_BAR:
            # STANDARD TITLE BAR
            self.setWindowFlags(Qt.FramelessWindowHint)
            self.setAttribute(Qt.WA_TranslucentBackground)

            # MOVE WINDOW / MAXIMIZE / RESTORE
            def moveWindow(event) -> None:
                # IF MAXIMIZED CHANGE TO NORMAL
                if UIFunctions.returnStatus(self):
                    UIFunctions.maximize_restore(self)
                # MOVE WINDOW
                if event.buttons() == Qt.LeftButton:
                    self.move(self.pos() + event.globalPos() - self.dragPos)
                    self.dragPos = event.globalPos()
                    event.accept()

            self.ui.headerContainer.mouseMoveEvent = moveWindow

            # CUSTOM GRIPS
            self.left_grip = CustomGrip(self, Qt.LeftEdge, True)
            self.right_grip = CustomGrip(self, Qt.RightEdge, True)
            self.top_grip = CustomGrip(self, Qt.TopEdge, True)
            self.bottom_grip = CustomGrip(self, Qt.BottomEdge, True)

        else:
            self.ui.appMargins.setContentsMargins(0, 0, 0, 0)
            self.ui.minimizeAppBtn.hide()
            self.ui.maximizeRestoreAppBtn.hide()
            self.ui.closeAppBtn.hide()
            self.ui.appSizeGrip.hide()

        # DROP SHADOW
        self.shadow = QGraphicsDropShadowEffect(self)
        self.shadow.setBlurRadius(17)
        self.shadow.setXOffset(0)
        self.shadow.setYOffset(0)
        self.shadow.setColor(QColor(0, 0, 0, 150))
        self.ui.bgApp.setGraphicsEffect(self.shadow)

        # RESIZE WINDOW
        self.sizegrip = QSizeGrip(self.ui.appSizeGrip)
        self.sizegrip.setStyleSheet(
            "width: 20px; height: 20px; margin 0px; padding: 0px;"
        )

        # MINIMIZE
        self.ui.headerContainer.minimizeAppBtn.clicked.connect(
            lambda: self.showMinimized()
        )

        # MAXIMIZE/RESTORE
        self.ui.headerContainer.maximizeRestoreAppBtn.clicked.connect(
            lambda: UIFunctions.maximize_restore(self)
        )

        # CLOSE APPLICATION
        self.ui.headerContainer.closeAppBtn.clicked.connect(lambda: self.close())

    def resize_grips(self) -> None:
        if Settings.App.ENABLE_CUSTOM_TITLE_BAR:
            self.left_grip.setGeometry(0, 10, 10, self.height())
            self.right_grip.setGeometry(self.width() - 10, 10, 10, self.height())
            self.top_grip.setGeometry(0, 0, self.width(), 10)
            self.bottom_grip.setGeometry(0, self.height() - 10, self.width(), 10)

    # ///////////////////////////////////////////////////////////////
    # END - GUI DEFINITIONS



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtGui import (
    QFont,
)
from PySide6.QtWidgets import (
    QSizePolicy,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# /////////////////////////////////////////////////////////////////////////////////////////////

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class Fonts:
    # Regular
    SEGOE_UI_8_REG = None
    SEGOE_UI_10_REG = None
    SEGOE_UI_12_REG = None

    # Semibold
    SEGOE_UI_8_SB = None
    SEGOE_UI_10_SB = None
    SEGOE_UI_12_SB = None

    # ///////////////////////////////////////////////////////////////
    def __init__(self) -> None:
        pass

    # ///////////////////////////////////////////////////////////////
    @classmethod
    def initFonts(cls) -> None:

        # Segoe UI - 8
        # ///////////////////////////////////////////////////////////////
        cls.SEGOE_UI_8_REG = QFont()
        cls.SEGOE_UI_8_REG.setFamily("Segoe UI")
        cls.SEGOE_UI_8_REG.setPointSize(8)
        cls.SEGOE_UI_8_REG.setBold(False)
        cls.SEGOE_UI_8_REG.setItalic(False)

        # Segoe UI - 10
        # ///////////////////////////////////////////////////////////////
        cls.SEGOE_UI_10_REG = QFont()
        cls.SEGOE_UI_10_REG.setFamily("Segoe UI")
        cls.SEGOE_UI_10_REG.setPointSize(10)
        cls.SEGOE_UI_10_REG.setBold(False)
        cls.SEGOE_UI_10_REG.setItalic(False)

        # Segoe UI - 12
        # ///////////////////////////////////////////////////////////////
        cls.SEGOE_UI_12_REG = QFont()
        cls.SEGOE_UI_12_REG.setFamily("Segoe UI")
        cls.SEGOE_UI_12_REG.setPointSize(12)
        cls.SEGOE_UI_12_REG.setBold(False)
        cls.SEGOE_UI_12_REG.setItalic(False)

        # Segoe UI Semibold - 8
        # ///////////////////////////////////////////////////////////////
        cls.SEGOE_UI_8_SB = QFont()
        cls.SEGOE_UI_8_SB.setFamily("Segoe UI Semibold")
        cls.SEGOE_UI_8_SB.setPointSize(8)
        cls.SEGOE_UI_8_SB.setBold(False)
        cls.SEGOE_UI_8_SB.setItalic(False)

        # Segoe UI Semibold - 10
        # ///////////////////////////////////////////////////////////////
        cls.SEGOE_UI_10_SB = QFont()
        cls.SEGOE_UI_10_SB.setFamily("Segoe UI Semibold")
        cls.SEGOE_UI_10_SB.setPointSize(10)
        cls.SEGOE_UI_10_SB.setBold(False)
        cls.SEGOE_UI_10_SB.setItalic(False)

        # Segoe UI Semibold - 12
        # ///////////////////////////////////////////////////////////////
        cls.SEGOE_UI_12_SB = QFont()
        cls.SEGOE_UI_12_SB.setFamily("Segoe UI Semibold")
        cls.SEGOE_UI_12_SB.setPointSize(12)
        cls.SEGOE_UI_12_SB.setBold(False)
        cls.SEGOE_UI_12_SB.setItalic(False)


# ///////////////////////////////////////////////////////////////
# ///////////////////////////////////////////////////////////////


class SizePolicy:
    # Horizontal priority
    H_EXPANDING_V_FIXED = None
    H_EXPANDING_V_PREFERRED = None

    # Vertical priority
    H_PREFERRED_V_EXPANDING = None

    # No priority
    H_EXPANDING_V_EXPANDING = None

    # ///////////////////////////////////////////////////////////////
    def __init__(self) -> None:
        pass

    # ///////////////////////////////////////////////////////////////
    @classmethod
    def initSizePolicy(cls) -> None:

        # Horizontal Expanding - Vertical Fixed
        # ///////////////////////////////////////////////////////////////
        cls.H_EXPANDING_V_FIXED = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        cls.H_EXPANDING_V_FIXED.setHorizontalStretch(0)
        cls.H_EXPANDING_V_FIXED.setVerticalStretch(0)

        # Horizontal Expanding - Vertical Preferred
        # ///////////////////////////////////////////////////////////////
        cls.H_EXPANDING_V_PREFERRED = QSizePolicy(
            QSizePolicy.Expanding, QSizePolicy.Preferred
        )
        cls.H_EXPANDING_V_PREFERRED.setHorizontalStretch(0)
        cls.H_EXPANDING_V_PREFERRED.setVerticalStretch(0)

        # Horizontal Preferred - Vertical Expanding
        # ///////////////////////////////////////////////////////////////
        cls.H_PREFERRED_V_EXPANDING = QSizePolicy(
            QSizePolicy.Preferred, QSizePolicy.Expanding
        )
        cls.H_PREFERRED_V_EXPANDING.setHorizontalStretch(0)
        cls.H_PREFERRED_V_EXPANDING.setVerticalStretch(0)

        # Horizontal Expanding - Vertical Expanding
        # ///////////////////////////////////////////////////////////////
        cls.H_EXPANDING_V_EXPANDING = QSizePolicy(
            QSizePolicy.Expanding, QSizePolicy.Expanding
        )
        cls.H_EXPANDING_V_EXPANDING.setHorizontalStretch(0)
        cls.H_EXPANDING_V_EXPANDING.setVerticalStretch(0)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Script pour créer des fichiers .qm dans le bon format Qt
Basé sur la documentation Qt et les exemples existants
"""

import xml.etree.ElementTree as ET
from pathlib import Path
import struct
import hashlib

def create_proper_qm_from_ts(ts_file_path: Path, qm_file_path: Path):
    """Crée un fichier .qm dans le bon format Qt"""
    
    print(f"🔄 Conversion de {ts_file_path.name} vers {qm_file_path.name}...")
    
    try:
        # Parser le fichier XML
        tree = ET.parse(ts_file_path)
        root = tree.getroot()
        
        # Extraire les traductions
        translations = {}
        for message in root.findall('.//message'):
            source = message.find('source')
            translation = message.find('translation')
            
            if source is not None and translation is not None:
                source_text = source.text.strip() if source.text else ""
                translation_text = translation.text.strip() if translation.text else ""
                
                if source_text and translation_text:
                    translations[source_text] = translation_text
        
        # Créer un fichier .qm dans le format Qt approprié
        create_qt_qm_file(qm_file_path, translations)
        
        print(f"✅ {len(translations)} traductions converties")
        return True
        
    except Exception as e:
        print(f"❌ Erreur lors de la conversion: {e}")
        return False

def create_qt_qm_file(qm_file_path: Path, translations: dict):
    """Crée un fichier .qm dans le format Qt approprié"""
    
    # Format Qt .qm basé sur la documentation et les exemples
    with open(qm_file_path, 'wb') as f:
        # En-tête Qt .qm
        # Magic number: "qm" suivi de 2 octets nuls
        f.write(b'qm\x00\x00')
        
        # Version (4 octets little-endian)
        f.write(struct.pack('<I', 0x01))
        
        # Nombre de traductions (4 octets little-endian)
        f.write(struct.pack('<I', len(translations)))
        
        # Écrire les traductions
        for source, translation in translations.items():
            # Encoder en UTF-8
            source_bytes = source.encode('utf-8')
            translation_bytes = translation.encode('utf-8')
            
            # Longueur de la source (4 octets little-endian)
            f.write(struct.pack('<I', len(source_bytes)))
            
            # Source
            f.write(source_bytes)
            
            # Longueur de la traduction (4 octets little-endian)
            f.write(struct.pack('<I', len(translation_bytes)))
            
            # Traduction
            f.write(translation_bytes)
        
        # Checksum (optionnel, mais souvent présent)
        f.write(struct.pack('<I', 0))

def main():
    """Fonction principale"""
    print("🔧 Création de fichiers .qm dans le bon format Qt")
    print("=" * 60)
    
    translations_dir = Path("ezqt_app/resources/translations")
    
    if not translations_dir.exists():
        print(f"❌ Dossier des traductions non trouvé: {translations_dir}")
        return
    
    # Trouver tous les fichiers .ts
    ts_files = list(translations_dir.glob("*.ts"))
    
    if not ts_files:
        print("❌ Aucun fichier .ts trouvé")
        return
    
    print(f"📄 Fichiers .ts trouvés: {len(ts_files)}")
    
    # Convertir chaque fichier .ts
    for ts_file in ts_files:
        qm_file = ts_file.with_suffix('.qm')
        if create_proper_qm_from_ts(ts_file, qm_file):
            print(f"✅ {qm_file.name} créé")
        else:
            print(f"❌ Échec de création de {qm_file.name}")
    
    print("\n🎉 Processus terminé !")
    print("📋 Prochaines étapes:")
    print("   1. Testez les nouveaux fichiers .qm")
    print("   2. Si ça ne fonctionne toujours pas, utilisez les .ts")



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

"""
EzQt_App package initialization.
"""
__version__ = "2.2.1"

# CLI
from .cli import main

# MAIN
from .main import init

# APP
from .app import EzQt_App, EzApplication

# KERNEL
from .kernel import Kernel, Settings, UIFunctions, Ui_MainWindow

# WIDGETS - Import specific classes to avoid circular imports
from .widgets.core.header import Header
from .widgets.core.menu import Menu
from .widgets.core.page_container import PageContainer
from .widgets.core.settings_panel import SettingsPanel



# -*- coding: utf-8 -*-
"""
Fonctions d'aide pour le système de traduction EzQt_App
"""

from .translation_manager import get_translation_manager

def tr(text: str) -> str:
    """
    Fonction de traduction globale simplifiée.
    Utilise le gestionnaire de traduction pour traduire un texte.
    
    Args:
        text (str): Texte à traduire
        
    Returns:
        str: Texte traduit ou texte original si pas de traduction
    """
    return get_translation_manager().translate(text)

def set_tr(widget, text: str):
    """
    Définit un texte traduit sur un widget et l'enregistre pour retraduction automatique.
    
    Args:
        widget: Widget Qt (QLabel, QPushButton, etc.)
        text (str): Texte original à traduire
    """
    get_translation_manager().set_translatable_text(widget, text)

def register_tr(widget, text: str):
    """
    Enregistre un widget pour retraduction automatique sans changer son texte immédiatement.
    
    Args:
        widget: Widget Qt
        text (str): Texte original à traduire
    """
    get_translation_manager().register_widget(widget, text)

def unregister_tr(widget):
    """
    Désenregistre un widget de la retraduction automatique.
    
    Args:
        widget: Widget Qt à désenregistrer
    """
    get_translation_manager().unregister_widget(widget)

def change_language(language_name: str) -> bool:
    """
    Change la langue de l'application.
    
    Args:
        language_name (str): Nom de la langue ("English", "Français", etc.)
        
    Returns:
        bool: True si le changement a réussi, False sinon
    """
    return get_translation_manager().load_language(language_name)

def get_available_languages() -> list:
    """
    Retourne la liste des langues disponibles.
    
    Returns:
        list: Liste des noms de langues disponibles
    """
    return get_translation_manager().get_available_languages()

def get_current_language() -> str:
    """
    Retourne la langue actuelle.
    
    Returns:
        str: Nom de la langue actuelle
    """
    return get_translation_manager().get_current_language_name() 


# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////
from PySide6.QtCore import (
    Qt,
)
from PySide6.QtGui import (
    QPainter,
    QPixmap,
    QIcon,
    QColor,
)

# IMPORT / GUI AND MODULES AND WIDGETS
# /////////////////////////////////////////////////////////////////////////////////////////////
from ...kernel.app_settings import Settings
from ...kernel.app_resources import *

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


class ThemeIcon(QIcon):
    def __init__(self, original_icon: QIcon) -> None:
        super().__init__()
        self.original_icon = (
            QIcon(original_icon) if isinstance(original_icon, str) else original_icon
        )
        self.updateIcon()
        # //////
        self._connect_theme_changed()

    # ///////////////////////////////////////////////////////////////

    def _connect_theme_changed(self):
        """Connect to theme changed signal using lazy import to avoid circular imports."""
        try:
            # Import lazy pour éviter l'import circulaire
            from ...widgets.core.ez_app import EzApplication
            EzApplication.instance().themeChanged.connect(self.updateIcon)
        except ImportError:
            # Fallback si l'import échoue
            print("Warning: Could not connect to EzApplication theme signal")

    # ///////////////////////////////////////////////////////////////

    def updateIcon(self) -> None:
        icon_color = "light" if Settings.Gui.THEME == "dark" else "dark"

        # Get the QPixmap from the original QIcon
        pixmap = self.original_icon.pixmap(self.original_icon.availableSizes()[0])

        # Create an image for manipulation
        image = pixmap.toImage()

        # Determine the new color
        new_color = QColor(
            Qt.white if icon_color == "light" else Qt.black
        )

        # Create a new QPixmap to draw the colored icon
        new_pixmap = QPixmap(image.size())
        new_pixmap.fill(Qt.transparent)

        # Draw the new colored icon
        painter = QPainter(new_pixmap)
        painter.drawImage(0, 0, image)
        painter.setCompositionMode(QPainter.CompositionMode_SourceIn)
        painter.fillRect(image.rect(), new_color)
        painter.end()

        self.addPixmap(new_pixmap)



# -*- coding: utf-8 -*-
# ///////////////////////////////////////////////////////////////

# IMPORT BASE
# ///////////////////////////////////////////////////////////////

# IMPORT SPECS
# ///////////////////////////////////////////////////////////////

# IMPORT / GUI AND MODULES AND WIDGETS
# ///////////////////////////////////////////////////////////////
from ezqt_app.helper import Helper
from ezqt_app.kernel import Kernel

## ==> GLOBALS
# ///////////////////////////////////////////////////////////////

## ==> VARIABLES
# ///////////////////////////////////////////////////////////////

## ==> CLASSES
# ///////////////////////////////////////////////////////////////


def init(mkTheme: bool = True) -> None:
    # //////
    # Configure UTF-8 encoding at system level
    import sys
    import locale
    import os
    
    # Set UTF-8 encoding for stdout/stderr
    if hasattr(sys.stdout, 'reconfigure'):
        sys.stdout.reconfigure(encoding='utf-8')
    if hasattr(sys.stderr, 'reconfigure'):
        sys.stderr.reconfigure(encoding='utf-8')
    
    # Set environment variables for UTF-8
    os.environ['PYTHONIOENCODING'] = 'utf-8'
    os.environ['QT_FONT_DPI'] = '96'
    
    # Set locale to UTF-8
    try:
        locale.setlocale(locale.LC_ALL, '')
    except locale.Error:
        # Fallback for systems without proper locale support
        pass
    
    # //////
    Kernel.checkAssetsRequirements()
    Kernel.makeRequiredFiles(mkTheme=mkTheme)
